/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information

*/


/*
    This is a simple test program which I've commented a little, I hope it
    makes some sense to somebody
*/


#include <stdio.h>
#include <time.h>

#include "redx.h"

/* defines */

#define RESOLUTION_X 320                    /* hor. resolution */
#define RESOLUTION_Y 240                    /* vert. resolution */
#define FULLSCRN FALSE                      /* full screen ? */

#define NUM_OBJECTS 10                      /* number of rotating objects */

#define NAME_BITMAP1 ".\\..\\MEDIA\\REDX.PCX"  /* name of first try for DX_LoadPcx */
#define NAME_BITMAP2 "REDX.PCX"             /* second try */

#define NAME_SOUND11 ".\\..\\MEDIA\\BOUNCE1.WAV" /* name of sounds, first try */
#define NAME_SOUND21 ".\\..\\MEDIA\\BOUNCE2.WAV"
#define NAME_SOUND31 ".\\..\\MEDIA\\BOUNCE3.WAV"

#define NAME_SOUND12 "BOUNCE1.WAV"          /* name of sounds, 2nd try */
#define NAME_SOUND22 "BOUNCE2.WAV"
#define NAME_SOUND32 "BOUNCE3.WAV"

#define NAME_FONT1 ".\\..\\MEDIA\\TERM12A.FNT" /* name of terminator font, first try */
#define NAME_FONT2 "TERM12A.FNT"            /* name of terminator font, second try */

#define NUM_SOUND_COPIES 3                  /* maximum number of simultaneously played sounds */

#define REDX_TEXT "PRESS ESC TO QUIT"

/* structures */

typedef struct _REDX
{
    FLOAT fVX, fVY;                         /* velocity for both axes (in pixels/20 ms) */
    DX_SPRITE *pdxsSprite;                  /* sprite (one for each object) */
} REDX;

/* global variables */

DWORD      dwBlack;                         /* this one will hold the color 'black' */

DX_BITMAP *pdxbRedx;                        /* pointer to the bitmap object holding all the
                                               frames of animation */
DX_BITMAP *pdxbText;                        /* pointer to hold the text bitmap */

REDX arrayRedx[NUM_OBJECTS];                /* array of objects */

DX_SOUND *arraySounds[3];                   /* 1 object for each sound */

DX_FONT *pdxfFont;                          /* pointer to hold the font */

DWORD dwRotateTicks;                        /* total number of ticks since last rotation */



/* REDXActivateApp : this (obligated) callback function is called when the app is
   activated or deactivated (check bActive), use this to reload the bitmaps,
   an app is (de)activated when the user presses alt-tab, handling this makes
   sure you can switch applications without breaking the flow of your game  */

void REDXActivateApp(DX_CONTEXT *pdxcContext)
{
    if (pdxcContext->bActive)
    {
        /* DXBitmap_Check checks if a bitmap is lost, and restores it if needed,
           returns TRUE when a restore happened */
        /* DXBitmap_Reload reloads our bitmap */
        if (DXBitmap_Check(pdxbRedx))
            DXBitmap_Reload(pdxbRedx);
        /* idem for font */
        if (DXFont_Check(pdxfFont))
            DXFont_Reload(pdxfFont);
        /* text bitmap needs to be redrawn if lost */
        if (DXBitmap_Check(pdxbText))
        {
            DXBitmap_Clear(pdxbText, dwBlack);
            DXBitmap_MakeTrans(pdxbText);
            DXFont_DrawString(pdxfFont, pdxbText, 0,0, REDX_TEXT);
        }
    }
}

/* REDXInit : this (obligated) callback function is called once when the app is
   initialized */

BOOL REDXInit(DX_CONTEXT *pdxcContext)
{
    int i;
    /* initialize the application, with it's classname & the caption
       of the main window*/
    if (!DX_Init(pdxcContext, "REDXTEST","RedX Test Application"))
    {
        DX_FatalError(pdxcContext,"DX_Init() failed");
        return FALSE;
    }
    /* set a FULLSCRN mode (FALSE) of RESOLUTION_XxRESOLUTION_Y */
    if (!DX_SetMode(pdxcContext, FULLSCRN, RESOLUTION_X, RESOLUTION_Y, 16))
    {
        DX_FatalError(pdxcContext,"DX_SetMode() failed");
        return FALSE;
    }
    /* initialize the sound module */
    if (!DX_InitSound(pdxcContext))
    {
        DX_FatalError(pdxcContext,"DX_InitSound() failed");
        return FALSE;
    }
    /* initialize the keyboard */
    if (!DX_InitKeyboard(pdxcContext))
    {
        DX_FatalError(pdxcContext,"DX_InitKeyboard() failed");
        return FALSE;
    }

    /* load the 3 sounds */
    if ((!(arraySounds[0] = DX_LoadDXSound(pdxcContext, NAME_SOUND11, NUM_SOUND_COPIES, TRUE)))&&
        (!(arraySounds[0] = DX_LoadDXSound(pdxcContext, NAME_SOUND12, NUM_SOUND_COPIES, TRUE))))
    {
        DX_FatalError(pdxcContext,"DX_LoadDXSound() failed");
        return FALSE;
    }
    if ((!(arraySounds[1] = DX_LoadDXSound(pdxcContext, NAME_SOUND21, NUM_SOUND_COPIES, TRUE)))&&
        (!(arraySounds[1] = DX_LoadDXSound(pdxcContext, NAME_SOUND22, NUM_SOUND_COPIES, TRUE))))
    {
        DX_FatalError(pdxcContext,"DX_LoadDXSound() failed");
        return FALSE;
    }
    if ((!(arraySounds[2] = DX_LoadDXSound(pdxcContext, NAME_SOUND31, NUM_SOUND_COPIES, TRUE)))&&
        (!(arraySounds[2] = DX_LoadDXSound(pdxcContext, NAME_SOUND32, NUM_SOUND_COPIES, TRUE))))
    {
        DX_FatalError(pdxcContext,"DX_LoadDXSound() failed");
        return FALSE;
    }
    
    /* load the bitmap, the third parameter tells RedX to set the palette of the Pcx if in 8 bit*/
    /* the fourth parameter lets RedX try to load it in vidememory */
    pdxbRedx = DX_LoadDXBitmapPcx(pdxcContext, NAME_BITMAP1, TRUE, TRUE);
    if (!pdxbRedx)
        pdxbRedx = DX_LoadDXBitmapPcx(pdxcContext, NAME_BITMAP2, TRUE, TRUE);
    if (!pdxbRedx)
    {
        DX_FatalError(pdxcContext,"DX_LoadDXBitmapPcx() failed");
        return FALSE;
    }

    /*  make it transparant */
    if (!DXBitmap_MakeTrans(pdxbRedx))
    {
        DX_FatalError(pdxcContext,"DXBitmap_MakeTrans() failed");
        return FALSE;
    }

    /* calculate the color 'black' for this mode/palette */
    dwBlack = DX_MakeColor(pdxcContext,0,0,0);

    /* try to load the font (in video mem, dark red) */
    pdxfFont = DX_LoadDXFont(pdxcContext, NAME_FONT1, 0xA0, 0x00, 0x00, TRUE);
    if (!pdxfFont)
        pdxfFont = DX_LoadDXFont(pdxcContext, NAME_FONT2, 0xA0, 0x00, 0x00, TRUE);
    if (!pdxfFont)
    {
        DX_FatalError(pdxcContext,"DX_LoadFont() failed");
        return FALSE;
    }

    /* create a bitmap to hold the text (so we only need to draw it once)*/
    pdxbText = DX_CreateDXBitmap(pdxcContext, DXFont_GetStringWidth(pdxfFont, REDX_TEXT),pdxfFont->dwHeight,TRUE);
    if (!pdxbText)
    {
        DX_FatalError(pdxcContext,"DX_CreateDXBitmap() failed");
        return FALSE;
    }
    /* clear the bitmap */
    DXBitmap_Clear(pdxbText,dwBlack);

    /* make it transparant */
    DXBitmap_MakeTrans(pdxbText);

    /* draw the text on it */
    DXFont_DrawString(pdxfFont, pdxbText, 0,0, REDX_TEXT);


    /* create all the sprite objects */
    for (i=0;i<NUM_OBJECTS;i++)
        if (!(arrayRedx[i].pdxsSprite = DX_CreateSpriteGrid(pdxcContext, pdxbRedx, 36, 6, 6)))
        {
            DX_FatalError(pdxcContext,"DX_CreateSpriteGrid() failed");
            return FALSE;
        }
    /* initialize the random number generator */
    srand((unsigned)time(NULL));

    /* generate for each object a random position, direction & speed */
    for (i=0;i<NUM_OBJECTS;i++)
    {
        arrayRedx[i].pdxsSprite->fX = (FLOAT)(1.0*rand()*(RESOLUTION_X-arrayRedx[i].pdxsSprite->dwWidth-1)/RAND_MAX);
        arrayRedx[i].pdxsSprite->fY = (FLOAT)(1.0*rand()*(RESOLUTION_Y-arrayRedx[i].pdxsSprite->dwHeight-1)/RAND_MAX);
        arrayRedx[i].fVX=0; arrayRedx[i].fVY=0;
        while ((arrayRedx[i].fVX==0)||(arrayRedx[i].fVY==0))
        {
            arrayRedx[i].fVX=(FLOAT)(1.0*rand()*2.0/RAND_MAX);
            arrayRedx[i].fVY=(FLOAT)(1.0*rand()*1.5/RAND_MAX);
        }
        arrayRedx[i].pdxsSprite->dwCurrentFrame = (DWORD)(1.0*rand()*31/RAND_MAX);
    }

    dwRotateTicks = 0; /* start counting the ticks */

    /* return TRUE if everything is OK */
    return TRUE;
} 

/* REDXLoop : obligated callback function, called as much as possible, as long
   as it returns TRUE. Can be used for drawing */
BOOL REDXLoop(DX_CONTEXT *pdxcContext)
{
    int i;
    DWORD dwTicks;

    /* only draw when the app is active */
    if (pdxcContext->bActive)
    {
        /* try to flip */
        if (!DXBitmap_Flip(pdxcContext->pdxbScreen))
            return TRUE; /* only draw if we could flip */
        /* clear the screen (backbuffer) */
        DXBitmap_Clear(pdxcContext->pdxbScreen, dwBlack);

        /* draw all the sprites */
        for (i=0;i<NUM_OBJECTS;i++)
            DXSprite_Draw(arrayRedx[i].pdxsSprite,pdxcContext->pdxbScreen);

        /* draw the text */
        DXBitmap_Blit(pdxbText, pdxcContext->pdxbScreen, 0, RESOLUTION_Y-pdxfFont->dwHeight);

        /* number of 'ticks' (1tick = 1ms) since last frame */
        dwTicks = DX_GetFrameTicks(pdxcContext);
        /* number of 'ticks' (1tick = 1ms) since last rotation */

        /* if time since last frame too long, normalize ticks  */
        if (dwTicks>100)
            dwTicks=100;


        dwRotateTicks += dwTicks;

        if (dwRotateTicks>20) /* rotate if needed */
        {
            dwRotateTicks -= 20;
            for (i=0;i<NUM_OBJECTS;i++) /* for all objects */
                DXSprite_NextFrame(arrayRedx[i].pdxsSprite); /* advance (rotate) sprite 1 frame */
        }

        for (i=0;i<NUM_OBJECTS;i++)
        {
            /* move the right number of pixels, depending on ticks & speed  */
            arrayRedx[i].pdxsSprite->fX += (FLOAT)(arrayRedx[i].fVX*(FLOAT)dwTicks/20.0);
            arrayRedx[i].pdxsSprite->fY += (FLOAT)(arrayRedx[i].fVY*(FLOAT)dwTicks/20.0);

        }        
        for (i=0;i<NUM_OBJECTS;i++)
        {
            /* if left or right border is crossed, turn around X-velocity (=bounce) */
            if ((arrayRedx[i].fVX<0)&&(arrayRedx[i].pdxsSprite->fX<=0)||
                (arrayRedx[i].fVX>0)&&(arrayRedx[i].pdxsSprite->fX>=RESOLUTION_X-arrayRedx[i].pdxsSprite->dwWidth))
            {
                DXSound_SetPan(arraySounds[i%3],arrayRedx[i].pdxsSprite->fX<=0?DSBPAN_LEFT:DSBPAN_RIGHT);
                DXSound_Play(arraySounds[i%3]); /* play sound 0, 1 or 2 */
                arrayRedx[i].fVX = -arrayRedx[i].fVX;
            }
            /* if top or bottom border is crossed, turn around Y-velocity (=bounce) */
            if ((arrayRedx[i].fVY<0)&&(arrayRedx[i].pdxsSprite->fY<=0)||
                (arrayRedx[i].fVY>0)&&(arrayRedx[i].pdxsSprite->fY>=RESOLUTION_Y-arrayRedx[i].pdxsSprite->dwHeight))
            {
                /* calculate sound position */
                LONG lPos = (LONG)(arrayRedx[i].pdxsSprite->fX)+(arrayRedx[i].pdxsSprite->dwWidth/2);
                lPos = (RESOLUTION_X/2-lPos)/(RESOLUTION_X/2)*DSBPAN_LEFT;
                /* set the position */
                DXSound_SetPan(arraySounds[i%3],lPos);
                DXSound_Play(arraySounds[i%3]); /* play sound 0, 1 or 2 */
                arrayRedx[i].fVY = -arrayRedx[i].fVY;
            }

        }
        /* poll the keyboard */
        DX_PollKeyboard(pdxcContext);
        /* was escape pressed ? --> quit */
        if (DX_KeyDown(pdxcContext, DIK_ESCAPE)&&(!(DX_KeyDown(pdxcContext, DIK_LCONTROL))))
        {
            return FALSE;
        }
    }
    /* continue */
    return TRUE;
}

/* REDXExit : obligated callback, called just before the appp exits, use this
   to clean up stuff */ 
BOOL REDXExit(DX_CONTEXT *pdxcContext)
{
    int i;
    /* free the bitmap surface */
    if (pdxbRedx)
        DXBitmap_Free(pdxbRedx);
    /* free all sprites */
    for(i=0;i<NUM_OBJECTS;i++)
    {
        if (arrayRedx[i].pdxsSprite)
            DXSprite_Free(arrayRedx[i].pdxsSprite);
        else
            break;
    }
    /* de-initialize the redx library */
    DX_Exit(pdxcContext);
    /* allways return TRUE */
    return TRUE;
}
