/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information
*/

#ifndef _REDX_H_
#define _REDX_H_

#include <windows.h>
#include <mmsystem.h>

#ifdef  MAKE_HRESULT
#undef  MAKE_HRESULT
#endif

#define MAKE_HRESULT(s,f,c) ((HRESULT)((((DWORD)(s))<<31)|(((DWORD)(f))<<16)|(((DWORD)(c)))))

#ifndef SUCCEEDED
#define SUCCEEDED(Status) ((HRESULT)(Status) >= 0)
#endif
#ifndef FAILED
#define FAILED(Status) ((HRESULT)(Status)<0)
#endif

#ifndef S_OK
#define S_OK                                   ((HRESULT)0x00000000L)
#define S_FALSE                                ((HRESULT)0x00000001L)
#endif

#ifndef SEVERITY_SUCCESS
#define SEVERITY_SUCCESS                       0
#define SEVERITY_ERROR                         1
#endif

#ifndef FACILITY_WIN32
#define FACILITY_WIN32                         7
#endif

#ifndef CO_E_NOTINITIALIZED
#define CO_E_NOTINITIALIZED                    _HRESULT_TYPEDEF_(0x800401F0L)
#endif

#ifndef REGDB_E_CLASSNOTREG
#define REGDB_E_CLASSNOTREG                    _HRESULT_TYPEDEF_(0x80040154L)
#endif

#ifndef E_ACCESSDENIED
#define E_ACCESSDENIED                         _HRESULT_TYPEDEF_(0x80070005L)
#endif


#define DXE_DDRAW                              1
#define DXE_DINPUT                             2
#define DXE_DSOUND                             3
#define DXE_DPLAY                              4

#define DD_ERROR                               pdxcContext->dwTypeError=DXE_DDRAW;
#define DI_ERROR                               pdxcContext->dwTypeError=DXE_DINPUT;
#define DS_ERROR                               pdxcContext->dwTypeError=DXE_DSOUND;
#define DP_ERROR                               pdxcContext->dwTypeError=DXE_DPLAY;

#define RETURN_DDE_FALSE {DD_ERROR return FALSE;}
#define RETURN_DIE_FALSE {DI_ERROR return FALSE;}
#define RETURN_DSE_FALSE {DS_ERROR return FALSE;}
#define RETURN_DPE_FALSE {DP_ERROR return FALSE;}

#include <ddraw.h>
#include <dinput.h>
#include <dsound.h>

#define LINK_GUID(name, l, w1, w2, b1, b2, b3, b4, b5, b6, b7, b8) \
       const GUID name \
                = { l, w1, w2, { b1, b2,  b3,  b4,  b5,  b6,  b7,  b8 } }

#ifndef MIN
#define MIN(x,y)     (((x) < (y)) ? (x) : (y))
#define MAX(x,y)     (((x) > (y)) ? (x) : (y))
#define MID(x,y,z)   MAX((x), MIN((y), (z)))
#endif

/*
    struct DX_BITMAP

    holds a surface (simple or with 1 backbuffer)
*/

struct DX_CONTEXT;

enum DXB_TYPE {user=0,resource,file,pcx};

typedef struct DX_BITMAP
{
    struct DX_CONTEXT * pdxcContext;        /* reference pointer to DX_CONTEXT */
    DWORD dwWidth;                          /* width */
    DWORD dwHeight;                         /* heigth */
    BOOL bTrans;                            /* set if the surface is transparent */
    LPDIRECTDRAWSURFACE lpDDSBack;          /* this is the one where all the blits go to */
    LPDIRECTDRAWSURFACE lpDDSPrimary;       /* only filled in case of a backbuffered surface */
    DDSURFACEDESC ddsd;                     /* to store the surface description */
    BOOL bHardFlip;                         /* set if hardware flip possible */
    RECT rectClient;                        /* client rectangle for soft flip */
    enum DXB_TYPE dxbType;                  /* type of bitmap */
    LPSTR lpName;                           /* the name of the bitmap */
} DX_BITMAP;

/*
    DX_CONTEXT :

    everything that's global for an application
*/

typedef struct DX_CONTEXT
{
    LPSTR lpAppName;                        /* name of the application & window class */
    LPSTR lpAppTitle;                       /* main window caption */
    HINSTANCE hinstApp;                     /* instance handle of the app */
    HWND hwndMain;                          /* handle of the main window */
    LPDIRECTDRAW lpDD;                      /* ddraw object */
    LPDIRECTINPUT lpDI;                     /* dinput object */
    LPDIRECTINPUTDEVICE lpDIKeyboardDevice; /* keyboard device */
    LPDIRECTDRAWPALETTE lpddpPalette;       /* the palette (if needed) */
    PALETTEENTRY aPe[256];                  /* the palette entry array */
    DWORD dwDepth;                          /* bit depth */
    DDPIXELFORMAT ddpf;                     /* pixel format */
    BYTE *pbRgb8;                           /* RGB conversion table for 8 bit modes */
    DX_BITMAP *pdxbScreen;                  /* screen bitmap */
    LPDIRECTSOUND lpDS;                     /* direct sound object */
    HRESULT hLastResult;                    /* last dd error */
    DWORD dwTypeError;                      /* 'type' off dd error */
    BOOL bFullScreen;                       /* true if fullscreen app */
    BOOL bActive;                           /* app has focus or not (alt-tab) */ 
    UINT uiTimerRes;                        /* resolution of the multimedia timer */
    DWORD dwTicks;                          /* frame time counter */
    void *pUserData;                        /* user-defined */
} DX_CONTEXT;

/*
    The 'lowest level' sound object, corresponding to a dsound secondary buffer
*/

typedef struct DX_SOUNDBUFFER
{
    struct DX_CONTEXT *pdxcContext;        /* reference pointer to DX_CONTEXT */
    LPDIRECTSOUNDBUFFER lpDSBuffer;        /* pointer to the buffer */
} DX_SOUNDBUFFER;

/*
    A sound object that can play multiple copies of the same sound simultaneously
*/

typedef struct DX_SOUND
{
    DWORD dwNumSounds;                     /* total number of sounds that can be played simultaneously */
    DWORD dwCurrentSound;                  /* current sound (next one played) */
    DX_SOUNDBUFFER **apdxsbSoundBuffer;    /* array of pointers to DX_SOUNDBUFFER objects */
} DX_SOUND;

/*
    This structure defines a sprite
*/

typedef struct DX_SPRITE
{
    DX_BITMAP *pdxbBitmap;                 /* bitmap on with all the frames of the sprite */
    RECT *pRectArray;                      /* one RECT for each frame */
    DWORD dwNumFrames;                     /* total number of frames */
    DWORD dwCurrentFrame;                  /* current frame */
    DWORD dwWidth;                         /* width of each frame */
    DWORD dwHeight;                        /* height of each frame */
    FLOAT fX;                              /* screen position x */
    FLOAT fY;                              /* "      "        y */
} DX_SPRITE;

typedef struct DX_FONT
{
    LPSTR lpFileName;
    DX_BITMAP *pdxbBitmap;                 /* bitmap containing all the characters */
    LPSTR lpFontName;                      /* name of the font file */
    RECT *arrayRect;                       /* one RECT for each character */
    DWORD dwBeginChar;                     /* ASCII code of the first character */
    DWORD dwEndChar;                       /* ASCII code of the last character */
    DWORD dwHeight;                        /* height of font in pixels */
    DWORD dwHSpacing;                      /* horizontal spacing when writing strings */
    BYTE bRed, bGreen, bBlue;              /* font base color */
} DX_FONT;

BOOL DX_Init(DX_CONTEXT *pdxContext, LPSTR lpAppName, LPSTR lpAppTitle);
BOOL DX_SetMode(DX_CONTEXT *pdxcContext, BOOL bFullScreen,
               DWORD dwWidth, DWORD dwHeight, DWORD dwDepth);
void DX_UnSetMode(DX_CONTEXT *pdxcContext);
BOOL DX_InitInput(DX_CONTEXT *pdxcContext);
void DX_Exit(DX_CONTEXT *pdxcContext);

DX_BITMAP *DX_CreateDXBitmap(DX_CONTEXT *pdxcContext, DWORD dwWidth, DWORD dwHeight, BOOL bVideo);
DX_BITMAP *DX_LoadDXBitmap(DX_CONTEXT *pdxcContext, LPSTR lpName, BOOL bVideo);

void DXBitmap_Reload(DX_BITMAP *pdxbBitmap);
void DXBitmap_Free(DX_BITMAP *pdxbBitmap);
BOOL DXBitmap_Check(DX_BITMAP *pdxbBitmap);
BOOL DXBitmap_MakeTrans(DX_BITMAP *pdxbBitmap);
BOOL DXBitmap_Clear(DX_BITMAP *pdxbBitmap, DWORD color);
BOOL DXBitmap_Flip(DX_BITMAP *pdxbBitmap);
BOOL DXBitmap_Blit(DX_BITMAP *pdxbFrom, DX_BITMAP *pdxbTo, LONG lX, LONG lY);
BOOL DXBitmap_BlitRect(DX_BITMAP *pdxbFrom, DX_BITMAP *pdxbTo, RECT *prectFrom, LONG lX, LONG lY);

BOOL DX_InitKeyboard(DX_CONTEXT *pdxcContext);
BOOL DX_PollKeyboard(DX_CONTEXT *pdxcContext);
BOOL DX_KeyDown(DX_CONTEXT *pdxcContext, int iKey);

BOOL DX_CreatePalette(DX_CONTEXT *pdxcContext);
BOOL DX_LoadPalette(DX_CONTEXT *pdxcContext, LPSTR pBitmapName);
BOOL DX_SetPalette(DX_CONTEXT *pdxcContext);

BOOL DX_InitTimer(DX_CONTEXT *pdxcContext, DWORD dwRes);
UINT DX_SetTimer(DX_CONTEXT *pdxcContext, UINT uiDelay, LPTIMECALLBACK lpTimeProc);
void DX_KillTimer(DX_CONTEXT *pdxcContext, UINT uiTimerId);
DWORD DX_GetFrameTicks(DX_CONTEXT *pdxcContext);

/* call back functions */
void REDXActivateApp(DX_CONTEXT *pdxcContext);
BOOL REDXInit(DX_CONTEXT *pdxcContext);
BOOL REDXLoop(DX_CONTEXT *pdxcContext);
BOOL REDXExit(DX_CONTEXT *pdxcContext);

BOOL DX_InitSound(DX_CONTEXT *pdxcContext);
DX_SOUNDBUFFER *DX_CreateDXSoundBuffer(DX_CONTEXT *pdxcContext, unsigned long size, WAVEFORMATEX* wfmt, BOOL bHardWare);
DX_SOUNDBUFFER *DX_LoadDXSoundBuffer(DX_CONTEXT *pdxcContext, LPCSTR lpSoundName, BOOL bHardWare);

DX_SOUNDBUFFER *DXSoundBuffer_Copy(DX_SOUNDBUFFER *pdxsbSoundBuffer);
void DXSoundBuffer_Free(DX_SOUNDBUFFER *pdxsbSoundBuffer);
BOOL DXSoundBuffer_Play(DX_SOUNDBUFFER *pdxsbSoundBuffer, BOOL bLooping);
BOOL DXSoundBuffer_Stop(DX_SOUNDBUFFER *pdxsbSoundBuffer);
BOOL DXSoundBuffer_Write(DX_SOUNDBUFFER *pdxsbSoundBuffer, HMMIO file, unsigned long size);
BOOL DXSoundBuffer_SetPan(DX_SOUNDBUFFER *pdxsbSoundBuffer, long val);
BOOL DXSoundBuffer_SetVolume(DX_SOUNDBUFFER *pdxsbSoundBuffer, LONG val);

DX_SOUND *DX_LoadDXSound(DX_CONTEXT *pdxcContext, LPCSTR lpSoundName, DWORD dwNumCopies, BOOL bHardWare);
BOOL DXSound_Play(DX_SOUND *pdxsSound);
BOOL DXSound_SetPan(DX_SOUND *pdxsSound, LONG lPan);
BOOL DXSound_Setvolume(DX_SOUND *pdxsSound, LONG lVolume);
void DXSound_Free(DX_SOUND *pdxsSound);


DX_SPRITE *DX_CreateSprite(DX_CONTEXT *pdxcContext, DX_BITMAP *pdxbBitmap, DWORD dwNumFrames);
DX_SPRITE *DX_CreateSpriteGrid(DX_CONTEXT *pdxcContext, DX_BITMAP *pdxbBitmap, DWORD dwNumFrames,
                               DWORD dwNumCols, DWORD dwNumRows);
BOOL DXSprite_MakeTrans(DX_SPRITE *pdxsSprite);
void DXSprite_Move(DX_SPRITE *pdxsSprite, FLOAT fX, FLOAT fY);
void DXSprite_SetFrame(DX_SPRITE *pdxsSprite, DWORD dwFrame);
void DXSprite_NextFrame(DX_SPRITE *pdxsSprite);
BOOL DXSprite_Draw(DX_SPRITE *pdxsSprite, DX_BITMAP *pdxbBitmap);
void DXSprite_Free(DX_SPRITE *pdxsSprite);

BOOL DX_CreateRgbTable8(DX_CONTEXT *pdxcContext);
BOOL DX_CreateRgbTable(DX_CONTEXT *pdxcContext);
DWORD DX_MakeColor8(DX_CONTEXT *pdxcContext, BYTE bR, BYTE bG, BYTE bB);
DWORD DX_MakeColor(DX_CONTEXT *pdxcContext, BYTE bR, BYTE bG, BYTE bB);

void DX_FatalError(DX_CONTEXT *pdxcContext, LPSTR szMessage);
LPSTR DX_GetDXErrorMsg(DX_CONTEXT *pdxcContext);
LPSTR DX_GetDDErrorMsg(DX_CONTEXT *pdxcContext);
LPSTR DX_GetDIErrorMsg(DX_CONTEXT *pdxcContext);
LPSTR DX_GetDSErrorMsg(DX_CONTEXT *pdxcContext);

BOOL DXBitmap_Lock(DX_BITMAP *pdxbBitmap);
BOOL DXBitmap_Unlock(DX_BITMAP *pdxbBitmap);
void DXBitmap_PutPixel(DX_BITMAP *pdxbBitmap, DWORD dwX, DWORD dwY, DWORD dwColor);

DX_BITMAP *DXBitmap_ReloadPcx(DX_BITMAP *pdxbBitmap, DX_CONTEXT *pdxcContext, LPSTR lpFilename,
                              BOOL bSetPalette, BOOL bVideo);
DX_BITMAP *DX_LoadDXBitmapPcx(DX_CONTEXT *pdxcContext, LPSTR lpFilename, BOOL bSetPalette, BOOL bVideo);


DX_FONT *DXFont_ReloadDXFont_(DX_FONT *pdxfFont, DX_CONTEXT *pdxcContext, LPSTR lpFilename,
                              BYTE bRed, BYTE bGreen, BYTE bBlue, BOOL bVideo);
DX_FONT *DX_LoadDXFont(DX_CONTEXT *pdxcContext, LPSTR lpFilename, BYTE bRed, BYTE bGreen,
                       BYTE bBlue, BOOL bVideo);
BOOL DXFont_Check(DX_FONT *pdxfFont);
void DXFont_Reload(DX_FONT *pdxfFont);
void DXFont_Free(DX_FONT *pdxfFont);
BOOL DXFont_DrawChar(DX_FONT *pdxfFont, DX_BITMAP *pdxbBitmap, INT iX, INT iY, INT iChar);
BOOL DXFont_DrawString(DX_FONT *pdxfFont, DX_BITMAP *pdxbBitmap, INT iX, INT iY, LPSTR lpText);
INT DXFont_GetCharWidth(DX_FONT *pdxfFont, INT iChar);
INT DXFont_GetStringWidth(DX_FONT *pdxfFont, LPSTR lpText);

#endif
