/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information

*/

#include "redx.h"

DX_BITMAP *DX_CreateDXBitmap(DX_CONTEXT *pdxcContext, DWORD dwWidth, DWORD dwHeight, BOOL bVideo)
{
	DDSURFACEDESC ddsd;
    DX_BITMAP *pdxbBitmap=NULL;

    if (!(pdxcContext->lpDD))
		return pdxbBitmap;

	ddsd.dwSize = sizeof(DDSURFACEDESC);
	ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.dwWidth  = dwWidth;
	ddsd.dwHeight = dwHeight;

	pdxbBitmap = (DX_BITMAP *)malloc(sizeof(DX_BITMAP));
    memset(pdxbBitmap,0,sizeof(DX_BITMAP));

    pdxbBitmap->pdxcContext = pdxcContext;
    
    pdxbBitmap->dwWidth = dwWidth;
    pdxbBitmap->dwHeight = dwHeight;

    if (bVideo)
    {
        ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN
            | DDSCAPS_VIDEOMEMORY;
        
        pdxcContext->hLastResult = IDirectDraw_CreateSurface(pdxcContext->lpDD, &ddsd,
            &(pdxbBitmap->lpDDSBack), NULL);
        if (SUCCEEDED(pdxcContext->hLastResult))
            goto DX_CreateBitmap_end;
    }
    
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN
        | DDSCAPS_SYSTEMMEMORY;
    
    pdxcContext->hLastResult = IDirectDraw_CreateSurface(pdxcContext->lpDD, &ddsd,
        &(pdxbBitmap->lpDDSBack), NULL);
    
    if (SUCCEEDED(pdxcContext->hLastResult))
        goto DX_CreateBitmap_end;
    
    DD_ERROR
    free(pdxbBitmap);
    pdxbBitmap=NULL;
    
DX_CreateBitmap_end:
	return pdxbBitmap;
}

DX_BITMAP *DX_LoadDXBitmap(DX_CONTEXT *pdxcContext, LPSTR lpName, BOOL bVideo)
{
    DX_BITMAP *pdxbBitmap=NULL;
    UINT uiFlags;
    HBITMAP hBitmap, hOldBitmap;
    DIBSECTION dibsection;
    HDC hdcBitmap, hdcSurface;
    BOOL bFile = FALSE;

	uiFlags = LR_CREATEDIBSECTION;

	if (!(hBitmap = (HBITMAP) LoadImage(pdxcContext->hinstApp, lpName, IMAGE_BITMAP, 0, 0, uiFlags)))
    {
        bFile = TRUE;
        uiFlags |= LR_LOADFROMFILE;
    }
    hBitmap = (HBITMAP) LoadImage(pdxcContext->hinstApp, lpName, IMAGE_BITMAP, 0, 0, uiFlags);

	if (!hBitmap)
		goto DX_LoadDXBitmap_end;

    if (!GetObject(hBitmap, sizeof(DIBSECTION), &dibsection))
        goto DX_LoadDXBitmap_end;

    if (!(pdxbBitmap=DX_CreateDXBitmap(pdxcContext, dibsection.dsBm.bmWidth, dibsection.dsBm.bmHeight, bVideo)))
	    goto DX_LoadDXBitmap_end;

    pdxbBitmap->dxbType = (bFile?file:resource);
	pdxbBitmap->lpName = (LPSTR)malloc(sizeof(char)*(strlen(lpName)+1));
	strcpy(pdxbBitmap->lpName, lpName);

	if (!(hdcBitmap = CreateCompatibleDC(0)))
    {
        DXBitmap_Free(pdxbBitmap);
        pdxbBitmap = NULL;
		goto DX_LoadDXBitmap_end;
	}

	if (!(hOldBitmap = (HBITMAP) SelectObject(hdcBitmap, hBitmap)))
    {
        DXBitmap_Free(pdxbBitmap);
        pdxbBitmap = NULL;
		goto DX_LoadDXBitmap_end;
	}

	pdxcContext->hLastResult = IDirectDrawSurface_GetDC(pdxbBitmap->lpDDSBack, &hdcSurface);
	if (FAILED(pdxcContext->hLastResult))
    {
        DXBitmap_Free(pdxbBitmap);
        pdxbBitmap = NULL;
        DD_ERROR
		goto DX_LoadDXBitmap_end;
	}

	if (!BitBlt(hdcSurface, 0, 0, pdxbBitmap->dwWidth, pdxbBitmap->dwHeight,
        hdcBitmap, 0, 0, SRCCOPY))
    {
        DXBitmap_Free(pdxbBitmap);
        pdxbBitmap = NULL;
		goto DX_LoadDXBitmap_end;
	}

	pdxcContext->hLastResult = IDirectDrawSurface_ReleaseDC(pdxbBitmap->lpDDSBack, hdcSurface);
	if (FAILED(pdxcContext->hLastResult))
    {
        DXBitmap_Free(pdxbBitmap);
        pdxbBitmap = NULL;
        DD_ERROR
		goto DX_LoadDXBitmap_end;
	}

	if (!(hOldBitmap = (HBITMAP) SelectObject(hdcBitmap, hOldBitmap)))
	{
        DXBitmap_Free(pdxbBitmap);
        pdxbBitmap = NULL;
		goto DX_LoadDXBitmap_end;
	}

	if (!DeleteDC(hdcBitmap)) {
        DXBitmap_Free(pdxbBitmap);
        pdxbBitmap = NULL;
		goto DX_LoadDXBitmap_end;
	}

	if (!DeleteObject(hBitmap))
    {
        DXBitmap_Free(pdxbBitmap);
        pdxbBitmap = NULL;
		goto DX_LoadDXBitmap_end;
	}

DX_LoadDXBitmap_end:
	return pdxbBitmap;
}

/* no tests are done here, because a bitmap should reload if it
   was loaded the first time (and maybe because I'm lazy) */
void DXBitmap_Reload(DX_BITMAP *pdxbBitmap)
{
    if ((pdxbBitmap->dxbType==file||pdxbBitmap->dxbType==resource))
    {
        UINT uiFlags;
        HBITMAP hBitmap, hOldBitmap;
        DIBSECTION dibsection;
        HDC hdcBitmap, hdcSurface;

	    uiFlags = LR_CREATEDIBSECTION;
        if (pdxbBitmap->dxbType==file)
            uiFlags |= LR_LOADFROMFILE;
	    hBitmap = (HBITMAP) LoadImage(pdxbBitmap->pdxcContext->hinstApp, pdxbBitmap->lpName, IMAGE_BITMAP, 0, 0, uiFlags);

        GetObject(hBitmap, sizeof(DIBSECTION), &dibsection);
	    hdcBitmap = CreateCompatibleDC(0);
	    hOldBitmap = (HBITMAP) SelectObject(hdcBitmap, hBitmap);
	    IDirectDrawSurface_GetDC(pdxbBitmap->lpDDSBack, &hdcSurface);

	    BitBlt(hdcSurface, 0, 0, pdxbBitmap->dwWidth, pdxbBitmap->dwHeight,
            hdcBitmap, 0, 0, SRCCOPY);

	    IDirectDrawSurface_ReleaseDC(pdxbBitmap->lpDDSBack, hdcSurface);
	    hOldBitmap = (HBITMAP) SelectObject(hdcBitmap, hOldBitmap);
	    DeleteDC(hdcBitmap);
	    DeleteObject(hBitmap);
    }
    else if (pdxbBitmap->dxbType==pcx)
    {
        DXBitmap_ReloadPcx(pdxbBitmap,NULL,NULL,FALSE,FALSE);
    }

	if (pdxbBitmap->bTrans)
	    DXBitmap_MakeTrans(pdxbBitmap);
}

void DXBitmap_Free(DX_BITMAP *pdxbBitmap)
{
	if (pdxbBitmap)
    {
        if (pdxbBitmap->lpName)
            free(pdxbBitmap->lpName);
        if (pdxbBitmap->lpDDSPrimary)
        {
            IDirectDrawSurface_Release(pdxbBitmap->lpDDSPrimary);
        }
        else if (pdxbBitmap->lpDDSBack)
        {
            IDirectDrawSurface_Release(pdxbBitmap->lpDDSBack);
        }
        free(pdxbBitmap);
    }
    return;
}

BOOL DXBitmap_Check(DX_BITMAP *pdxbBitmap)
{
    if (!pdxbBitmap)
        return FALSE;
    if (IDirectDrawSurface_IsLost(pdxbBitmap->lpDDSBack)==DDERR_SURFACELOST)
    {
        IDirectDrawSurface_Restore(pdxbBitmap->lpDDSBack);
        return TRUE;
    }
    return FALSE;
}


BOOL DXBitmap_Clear(DX_BITMAP *pdxbBitmap, DWORD color)
{
    DX_CONTEXT *pdxcContext;
    DDBLTFX ddBltFx;
    pdxcContext = pdxbBitmap->pdxcContext;
    ddBltFx.dwSize = sizeof(DDBLTFX);
    ddBltFx.dwFillColor = color;
    pdxcContext->hLastResult = IDirectDrawSurface_Blt(pdxbBitmap->lpDDSBack, NULL, NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &ddBltFx);
    if (FAILED(pdxbBitmap->pdxcContext->hLastResult))
        RETURN_DDE_FALSE
    return TRUE;
}

BOOL DXBitmap_MakeTrans(DX_BITMAP *pdxbBitmap)
{
    DX_CONTEXT *pdxcContext;
	DDSURFACEDESC ddsd;
	DDCOLORKEY ddck;
    UINT uiFlags;
    DWORD dwColor;

    if ((!pdxbBitmap)||(!(pdxbBitmap->lpDDSBack)))
        return FALSE;

    pdxcContext = pdxbBitmap->pdxcContext;

	ddsd.dwSize = sizeof(ddsd);

	uiFlags = DDLOCK_READONLY | DDLOCK_WAIT;
	pdxcContext->hLastResult = IDirectDrawSurface_Lock(pdxbBitmap->lpDDSBack, NULL, &ddsd, uiFlags, 0);
	if (FAILED(pdxcContext->hLastResult))
		goto DXBitmap_MakeTrans_end;

	dwColor = *(DWORD *)ddsd.lpSurface;
	dwColor &= (1 << ddsd.ddpfPixelFormat.dwRGBBitCount)-1;

	ddck.dwColorSpaceLowValue  = dwColor;
	ddck.dwColorSpaceHighValue = dwColor;

	pdxbBitmap->pdxcContext->hLastResult = IDirectDrawSurface_SetColorKey(pdxbBitmap->lpDDSBack, DDCKEY_SRCBLT, &ddck);

	if (FAILED(pdxcContext->hLastResult))
		goto DXBitmap_MakeTrans_end;

	pdxcContext->hLastResult = IDirectDrawSurface_Unlock(pdxbBitmap->lpDDSBack, ddsd.lpSurface);

	if (FAILED(pdxcContext->hLastResult))
		goto DXBitmap_MakeTrans_end;

    pdxbBitmap->bTrans = TRUE;

DXBitmap_MakeTrans_end:
    if (FAILED(pdxcContext->hLastResult))
        RETURN_DDE_FALSE
    return TRUE;
}

BOOL DXBitmap_Flip(DX_BITMAP *pdxbBitmap)
{
    RECT rect;
    DX_CONTEXT *pdxcContext;
    pdxcContext = pdxbBitmap->pdxcContext;
    if (pdxbBitmap->bHardFlip)
    {
        pdxcContext->hLastResult = IDirectDrawSurface_GetFlipStatus(pdxbBitmap->lpDDSPrimary, DDGFS_CANFLIP);
        if (SUCCEEDED(pdxcContext->hLastResult))
        {
            /* we can flip, so no check here */
            IDirectDrawSurface_Flip(pdxbBitmap->lpDDSPrimary,NULL, 0);
            return TRUE;
        }
        else
            RETURN_DDE_FALSE
    }
    else
    {
        rect.left=0;
        rect.top=0;
        rect.right=pdxbBitmap->dwWidth;
        rect.bottom=pdxbBitmap->dwHeight;
        pdxcContext->hLastResult = IDirectDrawSurface_Blt(pdxbBitmap->lpDDSPrimary,
            &(pdxbBitmap->rectClient), pdxbBitmap->lpDDSBack, &rect, DDBLT_WAIT, NULL);
        if (FAILED(pdxcContext->hLastResult))
            RETURN_DDE_FALSE
        return TRUE;
    }
}

BOOL DXBitmap_Blit(DX_BITMAP *pdxbFrom, DX_BITMAP *pdxbTo, LONG lX, LONG lY)
{
    RECT rectSource;

    rectSource.left = 0;
    rectSource.top = 0;
    rectSource.right = pdxbFrom->dwWidth;
    rectSource.bottom = pdxbFrom->dwHeight;


    return DXBitmap_BlitRect(pdxbFrom, pdxbTo, &rectSource, lX, lY);
}

BOOL DXBitmap_BlitRect(DX_BITMAP *pdxbFrom, DX_BITMAP *pdxbTo, RECT *prectFrom, LONG lX, LONG lY)
{
    HRESULT hLastResult;
    RECT rectSource;
    LONG lGap;
    DX_CONTEXT *pdxcContext;

    pdxcContext = pdxbTo->pdxcContext;

/*    memcpy(&rectSource,prectFrom,sizeof(RECT));*/
/*    which one is faster ? */
    rectSource.left = prectFrom->left;
    rectSource.top = prectFrom->top;
    rectSource.right = prectFrom->right;
    rectSource.bottom = prectFrom->bottom;
    
    if (lX<0)
    {
        rectSource.left -= lX;
        lX=0;
    }
    if (lY<0)
    {
        rectSource.top -= lY;
        lY=0;
    }
    if ((lGap=(rectSource.right-rectSource.left+lX-pdxbTo->dwWidth))>0)
        rectSource.right -= lGap;
    if ((lGap=(rectSource.bottom-rectSource.top+lY-pdxbTo->dwHeight))>0)
        rectSource.bottom -= lGap;

    hLastResult = IDirectDrawSurface_BltFast(pdxbTo->lpDDSBack, lX, lY, 
        pdxbFrom->lpDDSBack, &rectSource, DDBLTFAST_WAIT|(pdxbFrom->bTrans?DDBLTFAST_SRCCOLORKEY:0));

    if (FAILED(pdxcContext->hLastResult = hLastResult))
        RETURN_DDE_FALSE

    return TRUE;
}

