/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information

*/

#include "redx.h"

BOOL DX_CreateRgbTable8(DX_CONTEXT *pdxcContext)
{
    DWORD dwR, dwG, dwB;
    BYTE *pbRgb8;
    if (!(pdxcContext->pbRgb8))
    {
        pbRgb8 = (BYTE *)malloc(sizeof(BYTE)*(64*64*64));
    }
    else
    {
        pbRgb8 = pdxcContext->pbRgb8;
        pdxcContext->pbRgb8 = NULL;
    }
    if (!pbRgb8)
        return FALSE;
    for (dwB=0;dwB<64;dwB++)
        for (dwG=0;dwG<64;dwG++)
            for (dwR=0;dwR<64;dwR++)
                pbRgb8[dwR+(dwG*64)+(dwB*64*64)]=(BYTE)DX_MakeColor8(pdxcContext,(BYTE)(dwR*4),(BYTE)(dwG*4),(BYTE)(dwB*4));

    pdxcContext->pbRgb8 = pbRgb8;
    return TRUE;
}


BOOL DX_CreateRgbTable(DX_CONTEXT *pdxcContext)
{
    if (pdxcContext->dwDepth==8)
    {
        return DX_CreateRgbTable8(pdxcContext);
    }
    return TRUE;
}

DWORD DX_MakeColor8(DX_CONTEXT *pdxcContext, BYTE bR, BYTE bG, BYTE bB)
{
    PALETTEENTRY *pPalette;
    DWORD dwBestFit, dwFit, dwIndex, dwI;

    if (pdxcContext->pbRgb8)
    {
        return (DWORD)(pdxcContext->pbRgb8[bR/4+(bG/4*64)+(bB/4*64*64)]);
    }

    pPalette = pdxcContext->aPe;

    dwIndex = 0;
    dwBestFit = (bR-pPalette[0].peRed)*(bR-pPalette[0].peRed)+
                (bG-pPalette[0].peGreen)*(bG-pPalette[0].peGreen)+
                (bB-pPalette[0].peBlue)*(bB-pPalette[0].peBlue);

    for  (dwI=1;dwI<256;dwI++)
    {
        dwFit = (bR-pPalette[dwI].peRed)*(bR-pPalette[dwI].peRed)+
                (bG-pPalette[dwI].peGreen)*(bG-pPalette[dwI].peGreen)+
                (bB-pPalette[dwI].peBlue)*(bB-pPalette[dwI].peBlue);

        if (dwFit==0)
        {
            dwIndex = dwI;
            break;
        }

        if (dwFit < dwBestFit)
        {
            dwBestFit = dwFit;
            dwIndex = dwI;
        }
    }
    return dwIndex;
}

DWORD DX_MakeColor(DX_CONTEXT *pdxcContext, BYTE bR, BYTE bG, BYTE bB)
{
    if (pdxcContext->dwDepth==8)
        return DX_MakeColor8(pdxcContext, bR, bG, bB);
    else
    {
        DWORD dwValue;
        DDPIXELFORMAT *pDdpf = &(pdxcContext->ddpf);
        dwValue = ((pDdpf->dwRBitMask)*bR/0xFF)&pDdpf->dwRBitMask;
        dwValue |= ((pDdpf->dwGBitMask)*bG/0xFF)&pDdpf->dwGBitMask;
        dwValue |= ((pDdpf->dwBBitMask)*bB/0xFF)&pDdpf->dwBBitMask;
        return dwValue;
    }
}

