/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information

*/

#include <stdio.h>

#include "redx.h"

void DX_FatalError(DX_CONTEXT *pdxcContext, LPSTR szMessage)
{
    char szBuffer[1024];
    DX_UnSetMode(pdxcContext);
    ShowWindow(pdxcContext->hwndMain, SW_MINIMIZE);
    if (FAILED(pdxcContext->hLastResult))
        sprintf(szBuffer,"%s\nDirectX Error (%d) : %s",szMessage,pdxcContext->hLastResult,
          DX_GetDXErrorMsg(pdxcContext));
    else
        sprintf(szBuffer,"%s",szMessage);
    MessageBox(pdxcContext->hwndMain,szBuffer,pdxcContext->lpAppTitle,MB_OK|MB_ICONERROR);
}

LPSTR DX_GetDXErrorMsg(DX_CONTEXT *pdxcContext)
{
    if (!FAILED(pdxcContext->hLastResult))
        return NULL;
    switch(pdxcContext->dwTypeError)
    {
    case DXE_DDRAW:
        return DX_GetDDErrorMsg(pdxcContext);
        break;
    case DXE_DINPUT:
        return DX_GetDIErrorMsg(pdxcContext);
        break;
    case DXE_DSOUND:
        return DX_GetDSErrorMsg(pdxcContext);
        break;
    default:
        return "UNKNOWN DIRECTX ERROR TYPE";
    }
}

LPSTR DX_GetDDErrorMsg(DX_CONTEXT *pdxcContext)
{
    HRESULT h;
    if (!FAILED(pdxcContext->hLastResult))
        return NULL;
    h = pdxcContext->hLastResult;
    if (h==DDERR_ALREADYINITIALIZED)
        return "DDERR_ALREADYINITIALIZED";
    else if (h==DDERR_CANNOTATTACHSURFACE)
        return "DDERR_CANNOTATTACHSURFACE";
    else if (h==DDERR_CANNOTDETACHSURFACE)
        return "DDERR_CANNOTDETACHSURFACE";
    else if (h==DDERR_CURRENTLYNOTAVAIL)
        return "DDERR_CURRENTLYNOTAVAIL";
    else if (h==DDERR_EXCEPTION)
        return "DDERR_EXCEPTION";
    else if (h==DDERR_GENERIC)
        return "DDERR_GENERIC";
    else if (h==DDERR_HEIGHTALIGN)
        return "DDERR_HEIGHTALIGN";
    else if (h==DDERR_INCOMPATIBLEPRIMARY)
        return "DDERR_INCOMPATIBLEPRIMARY";
    else if (h==DDERR_INVALIDCAPS)
        return "DDERR_INVALIDCAPS";
    else if (h==DDERR_INVALIDCLIPLIST)
        return "DDERR_INVALIDCLIPLIST";
    else if (h==DDERR_INVALIDMODE)
        return "DDERR_INVALIDMODE";
    else if (h==DDERR_INVALIDOBJECT)
        return "DDERR_INVALIDOBJECT";
    else if (h==DDERR_INVALIDPARAMS)
        return "DDERR_INVALIDPARAMS";
    else if (h==DDERR_INVALIDPIXELFORMAT)
        return "DDERR_INVALIDPIXELFORMAT";
    else if (h==DDERR_INVALIDRECT)
        return "DDERR_INVALIDRECT";
    else if (h==DDERR_LOCKEDSURFACES)
        return "DDERR_LOCKEDSURFACES";
    else if (h==DDERR_NO3D)
        return "DDERR_NO3D";
    else if (h==DDERR_NOALPHAHW)
        return "DDERR_NOALPHAHW";
    else if (h==DDERR_NOCLIPLIST)
        return "DDERR_NOCLIPLIST";
    else if (h==DDERR_NOCOLORCONVHW)
        return "DDERR_NOCOLORCONVHW";
    else if (h==DDERR_NOCOOPERATIVELEVELSET)
        return "DDERR_NOCOOPERATIVELEVELSET";
    else if (h==DDERR_NOCOLORKEY)
        return "DDERR_NOCOLORKEY";
    else if (h==DDERR_NOCOLORKEYHW)
        return "DDERR_NOCOLORKEYHW";
    else if (h==DDERR_NODIRECTDRAWSUPPORT)
        return "DDERR_NODIRECTDRAWSUPPORT";
    else if (h==DDERR_NOEXCLUSIVEMODE)
        return "DDERR_NOEXCLUSIVEMODE";
    else if (h==DDERR_NOFLIPHW)
        return "DDERR_NOFLIPHW";
    else if (h==DDERR_NOGDI)
        return "DDERR_NOGDI";
    else if (h==DDERR_NOMIRRORHW)
        return "DDERR_NOMIRRORHW";
    else if (h==DDERR_NOTFOUND)
        return "DDERR_NOTFOUND";
    else if (h==DDERR_NOOVERLAYHW)
        return "DDERR_NOOVERLAYHW";
    else if (h==DDERR_NORASTEROPHW)
        return "DDERR_NORASTEROPHW";
    else if (h==DDERR_NOROTATIONHW)
        return "DDERR_NOROTATIONHW";
    else if (h==DDERR_NOSTRETCHHW)
        return "DDERR_NOSTRETCHHW";
    else if (h==DDERR_NOT4BITCOLOR)
        return "DDERR_NOT4BITCOLOR";
    else if (h==DDERR_NOT4BITCOLORINDEX)
        return "DDERR_NOT4BITCOLORINDEX";
    else if (h==DDERR_NOT8BITCOLOR)
        return "DDERR_NOT8BITCOLOR";
    else if (h==DDERR_NOTEXTUREHW)
        return "DDERR_NOTEXTUREHW";
    else if (h==DDERR_NOVSYNCHW)
        return "DDERR_NOVSYNCHW";
    else if (h==DDERR_NOZBUFFERHW)
        return "DDERR_NOZBUFFERHW";
    else if (h==DDERR_NOZOVERLAYHW)
        return "DDERR_NOZOVERLAYHW";
    else if (h==DDERR_OUTOFCAPS)
        return "DDERR_OUTOFCAPS";
    else if (h==DDERR_OUTOFMEMORY)
        return "DDERR_OUTOFMEMORY";
    else if (h==DDERR_OUTOFVIDEOMEMORY)
        return "DDERR_OUTOFVIDEOMEMORY";
    else if (h==DDERR_OVERLAYCANTCLIP)
        return "DDERR_OVERLAYCANTCLIP";
    else if (h==DDERR_OVERLAYCOLORKEYONLYONEACTIVE)
        return "DDERR_OVERLAYCOLORKEYONLYONEACTIVE";
    else if (h==DDERR_PALETTEBUSY)
        return "DDERR_PALETTEBUSY";
    else if (h==DDERR_COLORKEYNOTSET)
        return "DDERR_COLORKEYNOTSET";
    else if (h==DDERR_SURFACEALREADYATTACHED)
        return "DDERR_SURFACEALREADYATTACHED";
    else if (h==DDERR_SURFACEALREADYDEPENDENT)
        return "DDERR_SURFACEALREADYDEPENDENT";
    else if (h==DDERR_SURFACEBUSY)
        return "DDERR_SURFACEBUSY";
    else if (h==DDERR_CANTLOCKSURFACE)
        return "DDERR_CANTLOCKSURFACE";
    else if (h==DDERR_SURFACEISOBSCURED)
        return "DDERR_SURFACEISOBSCURED";
    else if (h==DDERR_SURFACELOST)
        return "DDERR_SURFACELOST";
    else if (h==DDERR_SURFACENOTATTACHED)
        return "DDERR_SURFACENOTATTACHED";
    else if (h==DDERR_TOOBIGHEIGHT)
        return "DDERR_TOOBIGHEIGHT";
    else if (h==DDERR_TOOBIGSIZE)
        return "DDERR_TOOBIGSIZE";
    else if (h==DDERR_TOOBIGWIDTH)
        return "DDERR_TOOBIGWIDTH";
    else if (h==DDERR_UNSUPPORTED)
        return "DDERR_UNSUPPORTED";
    else if (h==DDERR_UNSUPPORTEDFORMAT)
        return "DDERR_UNSUPPORTEDFORMAT";
    else if (h==DDERR_UNSUPPORTEDMASK)
        return "DDERR_UNSUPPORTEDMASK";
    else if (h==DDERR_VERTICALBLANKINPROGRESS)
        return "DDERR_VERTICALBLANKINPROGRESS";
    else if (h==DDERR_WASSTILLDRAWING)
        return "DDERR_WASSTILLDRAWING";
    else if (h==DDERR_XALIGN)
        return "DDERR_XALIGN";
    else if (h==DDERR_INVALIDDIRECTDRAWGUID)
        return "DDERR_INVALIDDIRECTDRAWGUID";
    else if (h==DDERR_DIRECTDRAWALREADYCREATED)
        return "DDERR_DIRECTDRAWALREADYCREATED";
    else if (h==DDERR_NODIRECTDRAWHW)
        return "DDERR_NODIRECTDRAWHW";
    else if (h==DDERR_PRIMARYSURFACEALREADYEXISTS)
        return "DDERR_PRIMARYSURFACEALREADYEXISTS";
    else if (h==DDERR_NOEMULATION)
        return "DDERR_NOEMULATION";
    else if (h==DDERR_REGIONTOOSMALL)
        return "DDERR_REGIONTOOSMALL";
    else if (h==DDERR_CLIPPERISUSINGHWND)
        return "DDERR_CLIPPERISUSINGHWND";
    else if (h==DDERR_NOCLIPPERATTACHED)
        return "DDERR_NOCLIPPERATTACHED";
    else if (h==DDERR_NOHWND)
        return "DDERR_NOHWND";
    else if (h==DDERR_HWNDSUBCLASSED)
        return "DDERR_HWNDSUBCLASSED";
    else if (h==DDERR_HWNDALREADYSET)
        return "DDERR_HWNDALREADYSET";
    else if (h==DDERR_NOPALETTEATTACHED)
        return "DDERR_NOPALETTEATTACHED";
    else if (h==DDERR_NOPALETTEHW)
        return "DDERR_NOPALETTEHW";
    else if (h==DDERR_BLTFASTCANTCLIP)
        return "DDERR_BLTFASTCANTCLIP";
    else if (h==DDERR_NOBLTHW)
        return "DDERR_NOBLTHW";
    else if (h==DDERR_NODDROPSHW)
        return "DDERR_NODDROPSHW";
    else if (h==DDERR_OVERLAYNOTVISIBLE)
        return "DDERR_OVERLAYNOTVISIBLE";
    else if (h==DDERR_NOOVERLAYDEST)
        return "DDERR_NOOVERLAYDEST";
    else if (h==DDERR_INVALIDPOSITION)
        return "DDERR_INVALIDPOSITION";
    else if (h==DDERR_NOTAOVERLAYSURFACE)
        return "DDERR_NOTAOVERLAYSURFACE";
    else if (h==DDERR_EXCLUSIVEMODEALREADYSET)
        return "DDERR_EXCLUSIVEMODEALREADYSET";
    else if (h==DDERR_NOTFLIPPABLE)
        return "DDERR_NOTFLIPPABLE";
    else if (h==DDERR_CANTDUPLICATE)
        return "DDERR_CANTDUPLICATE";
    else if (h==DDERR_NOTLOCKED)
        return "DDERR_NOTLOCKED";
    else if (h==DDERR_CANTCREATEDC)
        return "DDERR_CANTCREATEDC";
    else if (h==DDERR_NODC)
        return "DDERR_NODC";
    else if (h==DDERR_WRONGMODE)
        return "DDERR_WRONGMODE";
    else if (h==DDERR_IMPLICITLYCREATED)
        return "DDERR_IMPLICITLYCREATED";
    else if (h==DDERR_NOTPALETTIZED)
        return "DDERR_NOTPALETTIZED";
    else if (h==DDERR_UNSUPPORTEDMODE)
        return "DDERR_UNSUPPORTEDMODE";
    else if (h==DDERR_NOMIPMAPHW)
        return "DDERR_NOMIPMAPHW";
    else if (h==DDERR_INVALIDSURFACETYPE)
        return "DDERR_INVALIDSURFACETYPE";
    else if (h==DDERR_NOOPTIMIZEHW)
        return "DDERR_NOOPTIMIZEHW";
    else if (h==DDERR_NOTLOADED)
        return "DDERR_NOTLOADED";
    else if (h==DDERR_DCALREADYCREATED)
        return "DDERR_DCALREADYCREATED";
    else if (h==DDERR_NONONLOCALVIDMEM)
        return "DDERR_NONONLOCALVIDMEM";
    else if (h==DDERR_CANTPAGELOCK)
        return "DDERR_CANTPAGELOCK";
    else if (h==DDERR_CANTPAGEUNLOCK)
        return "DDERR_CANTPAGEUNLOCK";
    else if (h==DDERR_NOTPAGELOCKED)
        return "DDERR_NOTPAGELOCKED";
    else if (h==DDERR_MOREDATA)
        return "DDERR_MOREDATA";
    else if (h==DDERR_VIDEONOTACTIVE)
        return "DDERR_VIDEONOTACTIVE";
    else if (h==DDERR_DEVICEDOESNTOWNSURFACE)
        return "DDERR_DEVICEDOESNTOWNSURFACE";
    else if (h==DDERR_NOTINITIALIZED)
        return "DDERR_NOTINITIALIZED";
    else
        return "UNKNOWN DIRECTDRAW ERROR";
}

LPSTR DX_GetDIErrorMsg(DX_CONTEXT *pdxcContext)
{
    HRESULT h;
    if (!FAILED(pdxcContext->hLastResult))
        return NULL;
    h = pdxcContext->hLastResult;
    if (h==DIERR_OLDDIRECTINPUTVERSION)
        return "DIERR_OLDDIRECTINPUTVERSION";
    else if (h==DIERR_BETADIRECTINPUTVERSION)
        return "DIERR_BETADIRECTINPUTVERSION";
    else if (h==DIERR_BADDRIVERVER)
        return "DIERR_BADDRIVERVER";
    else if (h==DIERR_DEVICENOTREG)
        return "DIERR_DEVICENOTREG";
    else if (h==DIERR_NOTFOUND)
        return "DIERR_NOTFOUND";
/*  else if (h==e DIERR_OBJECTNOTFOUND)
        return "DIERR_OBJECTNOTFOUND";*/
    else if (h==DIERR_INVALIDPARAM)
        return "DIERR_INVALIDPARAM";
    else if (h==DIERR_NOINTERFACE)
        return "DIERR_NOINTERFACE";
    else if (h==DIERR_GENERIC)
        return "DIERR_GENERIC";
    else if (h==DIERR_OUTOFMEMORY)
        return "DIERR_OUTOFMEMORY";
    else if (h==DIERR_UNSUPPORTED)
        return "DIERR_UNSUPPORTED";
    else if (h==DIERR_NOTINITIALIZED)
        return "DIERR_NOTINITIALIZED";
    else if (h==DIERR_ALREADYINITIALIZED)
        return "DIERR_ALREADYINITIALIZED";
    else if (h==DIERR_NOAGGREGATION)
        return "DIERR_NOAGGREGATION";
    else if (h==DIERR_OTHERAPPHASPRIO)
        return "DIERR_OTHERAPPHASPRIO";
    else if (h==DIERR_INPUTLOST)
        return "DIERR_INPUTLOST";
    else if (h==DIERR_ACQUIRED)
        return "DIERR_ACQUIRED";
    else if (h==DIERR_NOTACQUIRED)
        return "DIERR_NOTACQUIRED";
/*  else if (h==e DIERR_READONLY)
        return "DIERR_READONLY";*/
/*  else if (h==e DIERR_HANDLEEXISTS)
        return "DIERR_HANDLEEXISTS";*/
    else if (h==DIERR_INSUFFICIENTPRIVS)
        return "DIERR_INSUFFICIENTPRIVS";
    else if (h==DIERR_DEVICEFULL)
        return "DIERR_DEVICEFULL";
    else if (h==DIERR_MOREDATA)
        return "DIERR_MOREDATA";
    else if (h==DIERR_NOTDOWNLOADED)
        return "DIERR_NOTDOWNLOADED";
    else if (h==DIERR_HASEFFECTS)
        return "DIERR_HASEFFECTS";
    else if (h==DIERR_NOTEXCLUSIVEACQUIRED)
        return "DIERR_NOTEXCLUSIVEACQUIRED";
    else if (h==DIERR_INCOMPLETEEFFECT)
        return "DIERR_INCOMPLETEEFFECT";
    else if (h==DIERR_NOTBUFFERED)
        return "DIERR_NOTBUFFERED";
    else if (h==DIERR_EFFECTPLAYING)
        return "DIERR_EFFECTPLAYING";
    else
        return "UNKNOWN DIRECTINPUT ERROR";
}

LPSTR DX_GetDSErrorMsg(DX_CONTEXT *pdxcContext)
{
    HRESULT h;
    if (!FAILED(pdxcContext->hLastResult))
        return NULL;
    h = pdxcContext->hLastResult;
    if (h==DSERR_ALLOCATED)
        return("DSERR_ALLOCATED");
    else if (h==DSERR_CONTROLUNAVAIL)
        return("DSERR_CONTROLUNAVAIL");
    else if (h==DSERR_INVALIDPARAM)
        return("DSERR_INVALIDPARAM");
    else if (h==DSERR_INVALIDCALL)
        return("DSERR_INVALIDCALL");
    else if (h==DSERR_GENERIC)
        return("DSERR_GENERIC");
    else if (h==DSERR_PRIOLEVELNEEDED)
        return("DSERR_PRIOLEVELNEEDED");
    else if (h==DSERR_OUTOFMEMORY)
        return("DSERR_OUTOFMEMORY");
    else if (h==DSERR_BADFORMAT)
        return("DSERR_BADFORMAT");
    else if (h==DSERR_UNSUPPORTED)
        return("DSERR_UNSUPPORTED");
    else if (h==DSERR_NODRIVER)
        return("DSERR_NODRIVER");
    else if (h==DSERR_ALREADYINITIALIZED)
        return("DSERR_ALREADYINITIALIZED");
    else if (h==DSERR_NOAGGREGATION)
        return("DSERR_NOAGGREGATION");
    else if (h==DSERR_BUFFERLOST)
        return("DSERR_BUFFERLOST");
    else if (h==DSERR_OTHERAPPHASPRIO)
        return("DSERR_OTHERAPPHASPRIO");
    else if (h==DSERR_UNINITIALIZED)
        return("DSERR_UNINITIALIZED");
    else if (h==DSERR_NOINTERFACE)
        return("DSERR_NOINTERFACE");
    else
        return "UNKNOWN DIRECTSOUND ERROR";
}

