/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information

*/

#include "redx.h"
#include "pack.h"

DX_FONT *DXFont_ReloadDXFont_(DX_FONT *pdxfFont, DX_CONTEXT *pdxcContext, LPSTR lpFileName,
                              BYTE bRed, BYTE bGreen, BYTE bBlue, BOOL bVideo)
{
    PACKFILE *f;
    INT iC, ch;
    INT iWidth, iHeight;
    DWORD dwBytesPerLine, dwX, dwY, dwNumPixels;
    DWORD dwPointer, dwTempPointer;
    BYTE *pbBuffer;
    DWORD dwColors[256], dwCurrentRect;
    RECT rectSource, rectDest[256];
    DX_BITMAP *pdxbBitmap;

    if (!pdxfFont)
    {
        pdxfFont = (DX_FONT *)malloc(sizeof(DX_FONT));
        memset((void *)pdxfFont,0,sizeof(DX_FONT));
        pdxfFont->lpFileName = (LPSTR)_strdup(lpFileName);
        pdxfFont->bRed = bRed;
        pdxfFont->bGreen = bGreen;
        pdxfFont->bBlue = bBlue;
    }
    else
    {
        pdxcContext = pdxfFont->pdxbBitmap->pdxcContext;
        lpFileName = pdxfFont->lpFileName;
        bRed = pdxfFont->bRed;
        bGreen = pdxfFont->bGreen;
        bBlue = pdxfFont->bBlue;
    }

    f = pack_fopen(lpFileName, F_READ);
    if (!f)
    {
        free (pdxfFont);
        return NULL;
    }
    
    pack_getc(f);                    /* skip manufacturer ID */
    pack_getc(f);                    /* skip version flag */
    pack_getc(f);                    /* skip encoding flag */
    
    if (pack_getc(f) != 8)           /* we can only handle 8 bits per pixel */
    {
        pack_fclose(f);
        free (pdxfFont);
        return NULL;
    }

    iWidth = -(pack_igetw(f));        /* xmin */
    iHeight = -(pack_igetw(f));       /* ymin */
    iWidth += pack_igetw(f) + 1;      /* xmax */
    iHeight += pack_igetw(f) + 1;     /* ymax */
    

    pack_igetl(f);                   /* skip DPI values */
    
    for (iC=0; iC<16; iC++)          /* skip the 16 color pallete */
    {
        pack_getc(f);
        pack_getc(f);
        pack_getc(f);
    }
    
    pack_getc(f);
    
    if (pack_getc(f) != 1)           /* must be a 256 color file */
    {
        pack_fclose(f);
        free (pdxfFont);
        return NULL;
    }
    
    dwBytesPerLine = (DWORD)pack_igetw(f);
    
    for (iC=0; iC<60; iC++)          /* skip some more junk */
        pack_getc(f);

    if (!(pbBuffer = (BYTE *)malloc(sizeof(BYTE)*iWidth*iHeight))) /* allocate buffer */
    {
        pack_fclose(f);
        free (pdxfFont);
        return NULL;
    }

    for (dwY=0; dwY<(DWORD)iHeight; dwY++)       /* put RLE encoded PCX data in memory buffer */
    {
        dwX = 0;
        
        while (dwX < dwBytesPerLine)
        {
            ch = pack_getc(f);
            if ((ch & 0xC0) == 0xC0)
            {
                iC = (ch & 0x3F);
                ch = pack_getc(f);
            }
            else
                iC = 1;
            
            while (iC--)
                pbBuffer[dwY*iWidth+dwX++]=(BYTE)ch;
        }
    }
    
    while (!pack_feof(f))           /* skip the 256 color pallete */
    {
        if (pack_getc(f)==12)
        {
            for (iC=0; iC<256; iC++)
            {
                pack_getc(f);
                pack_getc(f);
                pack_getc(f);
            }
            break;
        }
    }
    
    pack_fclose(f);
    
    if (errno)
    {
        free (pdxfFont);
        free (pbBuffer);
        return NULL;
    }

    /* create a temporary system bitmap (if it isn't a reload) */
    if (!pdxfFont->pdxbBitmap)
    {
        if (!(pdxbBitmap = DX_CreateDXBitmap(pdxcContext,(DWORD)iWidth,(DWORD)iHeight,FALSE)))
        {
            free (pdxfFont);
            free (pbBuffer);
            return NULL;
        }
    }
    else /* otherwise, we will write directly to the font bitmap */
        pdxbBitmap = pdxfFont->pdxbBitmap;

    /* calculate all possible colors : */

    for (iC=0;iC<256;iC++)
        dwColors[iC] = DX_MakeColor(pdxcContext,(BYTE)(bRed*iC/0xFF),(BYTE)(bGreen*iC/0xFF),(BYTE)(bBlue*iC/0xFF));


    /* lock the system bitmap */
    if (!DXBitmap_Lock(pdxbBitmap))
    {
        DXBitmap_Free(pdxbBitmap);
        free (pdxfFont);
        free (pbBuffer);
        return NULL;
    }

    
    /* copy all the characters in the temporary bitmap */
    dwPointer = 0;
    dwNumPixels = iWidth*iHeight;
    dwCurrentRect = 0;
    rectDest[dwCurrentRect].left = 0;
    rectDest[dwCurrentRect].top = 0;
    while (dwPointer<dwNumPixels)
    {
        /* search the next char */
        while ((dwPointer<dwNumPixels)&&(pbBuffer[dwPointer]==0xFF))
            dwPointer++;
        
        if (dwPointer>=dwNumPixels)
            break;

        /* left & right edge of character */
        dwTempPointer = dwPointer;
        rectSource.left = dwX = dwPointer%(DWORD)iWidth;
        while ((dwX<(DWORD)iWidth)&&(pbBuffer[dwPointer]!=0xFF))
        {
            dwX++;
            dwPointer++;
        }
        rectSource.right = dwX;
        /* reset pointer */
        dwPointer = dwTempPointer;

        /* top & bottom edge of character */
        dwTempPointer = dwPointer;
        rectSource.top = dwY = dwPointer/(DWORD)iWidth;
        while ((dwY<(DWORD)iHeight)&&(pbBuffer[dwPointer]!=0xFF))
        {
            dwY++;
            dwPointer+=(DWORD)(iWidth);
        }
        rectSource.bottom = dwY;
        /* reset pointer */
        dwPointer = dwTempPointer;

        /* destination rectangle over the edge ? start next line */
        if ((rectDest[dwCurrentRect].left+(rectSource.right-rectSource.left))>=iWidth)
        {
            rectDest[dwCurrentRect].left = 0;
            rectDest[dwCurrentRect].top += (rectSource.bottom-rectSource.top);
        }

        /* copy the character, and delete it from the buffer  */        
        for (dwY=0;dwY<(DWORD)(rectSource.bottom-rectSource.top);dwY++)
        {
            for (dwX=0;dwX<(DWORD)(rectSource.right-rectSource.left);dwX++)
            {
                DXBitmap_PutPixel(pdxbBitmap,
                    rectDest[dwCurrentRect].left+dwX,rectDest[dwCurrentRect].top+dwY,
                    dwColors[pbBuffer[rectSource.left+dwX+(rectSource.top+dwY)*(DWORD)iWidth]]);
                pbBuffer[rectSource.left+dwX+(rectSource.top+dwY)*(DWORD)iWidth] = 0xFF;
            }
        }
        /* fill in the rest of the destination rectangle */
        rectDest[dwCurrentRect].right = rectDest[dwCurrentRect].left+dwX;
        rectDest[dwCurrentRect].bottom = rectDest[dwCurrentRect].top+dwY;

        /* next destination rectangle */
        dwCurrentRect++;
        rectDest[dwCurrentRect].top = rectDest[dwCurrentRect-1].top;
        rectDest[dwCurrentRect].left = rectDest[dwCurrentRect-1].left+dwX;
    }
    
    /* we've finished writing directly to the bitmap */
    DXBitmap_Unlock(pdxbBitmap);

    /* we can dump the buffer now */
    free(pbBuffer);

    /* now create the real bitmap, if it doesn't exist yet (this step is skipped in a reload) */
    if (!pdxfFont->pdxbBitmap)
    {
        pdxfFont->pdxbBitmap = DX_CreateDXBitmap(pdxcContext,(DWORD)iWidth,rectDest[dwCurrentRect-1].bottom,bVideo);
        if (!(pdxfFont->pdxbBitmap))
        {
            DXBitmap_Free(pdxbBitmap);
            free (pdxfFont);
            return NULL;
        }
        /* blit the system bitmap to the 'real one' */
        rectSource.left = 0;
        rectSource.top = 0;
        rectSource.right = (DWORD)iWidth;
        rectSource.bottom = rectDest[dwCurrentRect-1].bottom;
        DXBitmap_BlitRect(pdxbBitmap,pdxfFont->pdxbBitmap,&rectSource,0,0);

        /* free the sytem bitmap */
        DXBitmap_Free(pdxbBitmap);

    }
    /* make the bitmap transparant */
    DXBitmap_MakeTrans(pdxfFont->pdxbBitmap);


    /* info about the range of characters */
    pdxfFont->dwBeginChar = 32;
    pdxfFont->dwEndChar = pdxfFont->dwBeginChar+dwCurrentRect-1;

    /* font height */
    pdxfFont->dwHeight = rectDest[dwCurrentRect-1].bottom-rectDest[dwCurrentRect-1].top;

    /* fill the array of rectangles */
    if (!(pdxfFont->arrayRect))
    {
        pdxfFont->arrayRect = (RECT *)malloc(sizeof(RECT)*dwCurrentRect);
        for (iC=0;iC<(INT)dwCurrentRect;iC++)
            pdxfFont->arrayRect[iC] = rectDest[iC];
    }

    return pdxfFont;
}

DX_FONT *DX_LoadDXFont(DX_CONTEXT *pdxcContext, LPSTR lpFilename, BYTE bRed, BYTE bGreen, BYTE bBlue, BOOL bVideo)
{
    return DXFont_ReloadDXFont_(NULL, pdxcContext, lpFilename, bRed, bGreen, bBlue, bVideo);
}

BOOL DXFont_Check(DX_FONT *pdxfFont)
{
    if (!pdxfFont)
        return FALSE;
    return DXBitmap_Check(pdxfFont->pdxbBitmap);
}

/* no return value, if it loaded the first time, it should still work */

void DXFont_Reload(DX_FONT *pdxfFont)
{
    DXFont_ReloadDXFont_(pdxfFont, NULL, NULL, 0, 0, 0, FALSE);
}

void DXFont_Free(DX_FONT *pdxfFont)
{
    if (pdxfFont)
    {
        if (pdxfFont->pdxbBitmap)
            DXBitmap_Free(pdxfFont->pdxbBitmap);
        if (pdxfFont->arrayRect)
            free(pdxfFont->arrayRect);
        if (pdxfFont->lpFileName)
            free(pdxfFont->lpFileName);
    }
}

BOOL DXFont_DrawChar(DX_FONT *pdxfFont, DX_BITMAP *pdxbBitmap, INT iX, INT iY, INT iChar)
{
    if (!pdxfFont)
        return FALSE;
    if (iChar<(INT)(pdxfFont->dwBeginChar))
        iChar = (INT)(pdxfFont->dwBeginChar);
    if (iChar>(INT)(pdxfFont->dwEndChar))
        iChar = (INT)(pdxfFont->dwBeginChar);
    iChar -= (INT)(pdxfFont->dwBeginChar);
    return DXBitmap_BlitRect(pdxfFont->pdxbBitmap,pdxbBitmap,&(pdxfFont->arrayRect[iChar]),iX,iY);
}

BOOL DXFont_DrawString(DX_FONT *pdxfFont, DX_BITMAP *pdxbBitmap, INT iX, INT iY, LPSTR lpText)
{
    INT iChar, i=0;
    if (!pdxfFont)
        return FALSE;
    while (iChar = lpText[i++])
    {
        if (iChar<(INT)(pdxfFont->dwBeginChar))
            iChar = (INT)(pdxfFont->dwBeginChar);
        if (iChar>(INT)(pdxfFont->dwEndChar))
            iChar = (INT)(pdxfFont->dwBeginChar);
        iChar -= (INT)(pdxfFont->dwBeginChar);
        if (!(DXBitmap_BlitRect(pdxfFont->pdxbBitmap,pdxbBitmap,&(pdxfFont->arrayRect[iChar]),iX,iY)))
            return FALSE;
        iX += (pdxfFont->arrayRect[iChar].right-pdxfFont->arrayRect[iChar].left)+pdxfFont->dwHSpacing;
    }
    return TRUE;
}

INT DXFont_GetCharWidth(DX_FONT *pdxfFont, INT iChar)
{
    if (!pdxfFont)
        return -1;
    if (iChar<(INT)(pdxfFont->dwBeginChar))
        iChar = (INT)(pdxfFont->dwBeginChar);
    if (iChar>(INT)(pdxfFont->dwEndChar))
        iChar = (INT)(pdxfFont->dwBeginChar);
    iChar -= (INT)(pdxfFont->dwBeginChar);
    return (pdxfFont->arrayRect[iChar].right-pdxfFont->arrayRect[iChar].left);
}

INT DXFont_GetStringWidth(DX_FONT *pdxfFont, LPSTR lpText)
{
    INT iChar, iWidth = 0, i=0;
    if (!pdxfFont)
        return -1;
    while (iChar = lpText[i++])
    {
        if (iChar<(INT)(pdxfFont->dwBeginChar))
            iChar = (INT)(pdxfFont->dwBeginChar);
        if (iChar>(INT)(pdxfFont->dwEndChar))
            iChar = (INT)(pdxfFont->dwBeginChar);
        iChar -= (INT)(pdxfFont->dwBeginChar);
        iWidth += (pdxfFont->arrayRect[iChar].right-pdxfFont->arrayRect[iChar].left);
    }
    return iWidth;
}

