/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information

*/

#include "redx.h"

BOOL DX_CreatePalette(DX_CONTEXT *pdxcContext)
{
    DWORD dwI;

    PALETTEENTRY *pPaletteEntry;
    pPaletteEntry = pdxcContext->aPe;

    if (pdxcContext->lpddpPalette)
    {
        IDirectDrawPalette_Release(pdxcContext->lpddpPalette);
        pdxcContext->lpddpPalette = NULL;
    }

    if (!(pdxcContext->bFullScreen))
    {
        /* set up the palette entries for windowed mode */
        /* set up the windows static entries */
        for (dwI = 0; dwI < 10 ; dwI++)
        {
            /* the first 10 static entries */
            pPaletteEntry[dwI].peFlags = PC_EXPLICIT;
            pPaletteEntry[dwI].peRed = (BYTE)dwI;
            pPaletteEntry[dwI].peGreen = 0;
            pPaletteEntry[dwI].peBlue = 0;
            
            /* the last 10 static entries */
            pPaletteEntry[dwI+246].peFlags = PC_EXPLICIT;
            pPaletteEntry[dwI+246].peRed = (BYTE)(dwI+246);
            pPaletteEntry[dwI+246].peGreen = 0;
            pPaletteEntry[dwI+246].peBlue = 0;
        }
        
        /* now set up the private entries */
        for (dwI = 10; dwI < 246; dwI ++)
        {
            pPaletteEntry[dwI].peFlags = PC_NOCOLLAPSE|PC_RESERVED;
            pPaletteEntry[dwI].peRed = (BYTE)(((dwI >> 5) & 0x07) * 255 / 7);
            pPaletteEntry[dwI].peGreen = (BYTE)(((dwI >> 2) & 0x07) * 255 / 7);
            pPaletteEntry[dwI].peBlue = (BYTE)(((dwI >> 0) & 0x03) * 255 / 3);
        }
        
    }
    else
    {
        /* build a 332 palette as the default in fullscreen mode */
        for (dwI=0; dwI<256; dwI++)
        {
            pPaletteEntry[dwI].peRed   = (BYTE)(((dwI >> 5) & 0x07) * 255 / 7);
            pPaletteEntry[dwI].peGreen = (BYTE)(((dwI >> 2) & 0x07) * 255 / 7);
            pPaletteEntry[dwI].peBlue  = (BYTE)(((dwI >> 0) & 0x03) * 255 / 3);
            pPaletteEntry[dwI].peFlags = (BYTE)0;
        }
    }

    pdxcContext->hLastResult = IDirectDraw_CreatePalette(pdxcContext->lpDD, DDPCAPS_8BIT|DDPCAPS_ALLOW256, pdxcContext->aPe, &(pdxcContext->lpddpPalette), NULL);
    if (FAILED(pdxcContext->hLastResult))
        RETURN_DDE_FALSE
    /* now set the palette */
    pdxcContext->hLastResult = IDirectDrawSurface_SetPalette(pdxcContext->pdxbScreen->lpDDSPrimary, pdxcContext->lpddpPalette);
    if (FAILED(pdxcContext->hLastResult))
        RETURN_DDE_FALSE

    if (pdxcContext->bFullScreen)
    {
        /* fullscreen : we set the 332 palette */
        pdxcContext->hLastResult = IDirectDrawPalette_SetEntries(pdxcContext->lpddpPalette, 0, 0, 256, pdxcContext->aPe);
    }
    else
    {
        HDC hdc;
        /* windowed : we set the private part of the palette */
        pdxcContext->hLastResult = IDirectDrawPalette_SetEntries(pdxcContext->lpddpPalette, 0, 10, 236, &(pdxcContext->aPe[10]));
        if (FAILED(pdxcContext->hLastResult))
            RETURN_DDE_FALSE
        /* and now we get the static entries */
        hdc=GetDC(0);
        GetSystemPaletteEntries(hdc,0,10,pdxcContext->aPe);
        GetSystemPaletteEntries(hdc,246,10,&(pdxcContext->aPe[246]));
        ReleaseDC(0,hdc);
    }

    return TRUE;
}

BOOL DX_LoadPalette(DX_CONTEXT *pdxcContext, LPSTR pBitmapName)
{
    int i, n, s, e;
    HANDLE handle;
    LPBITMAPINFOHEADER lpbi;
    RGBQUAD *pRgb;
    DWORD dwRead;

    /* make sure the palette is created */
    DX_CreatePalette(pdxcContext);
        
    /* get a pointer to the bitmap resource */
    if (pBitmapName && (handle = FindResource(NULL, pBitmapName, RT_BITMAP)))
    {
        lpbi = (LPBITMAPINFOHEADER)LockResource(LoadResource(NULL, handle));
        if (!lpbi)
            ; /* LockResource failed */
        pRgb = (RGBQUAD*)((BYTE*)lpbi + lpbi->biSize);
        
        if (lpbi == NULL || lpbi->biSize < sizeof(BITMAPINFOHEADER))
            n = 0;
        else if (lpbi->biBitCount > 8)
            n = 0;
        else if (lpbi->biClrUsed == 0)
            n = 1 << lpbi->biBitCount;
        else
            n = lpbi->biClrUsed;

        s = 0; e = n;
        if ((!pdxcContext->bFullScreen)&&(n==256))
        {
            s=10; e=246;
        }
        for(i=s; i<e; i++ )
        {
            pdxcContext->aPe[i].peRed   = pRgb[i].rgbRed;
            pdxcContext->aPe[i].peGreen = pRgb[i].rgbGreen;
            pdxcContext->aPe[i].peBlue  = pRgb[i].rgbBlue;
        }
    }
    else if (pBitmapName && (handle = CreateFile(pBitmapName, GENERIC_READ, FILE_SHARE_READ, 
        NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 
        NULL)) != INVALID_HANDLE_VALUE)
    {
        BITMAPFILEHEADER bf;
        BITMAPINFOHEADER bi;
        PALETTEENTRY aPe[256];

        ReadFile(handle, &bf, sizeof(bf), &dwRead, NULL);
        ReadFile(handle, &bi, sizeof(bi), &dwRead, NULL);
        ReadFile(handle, aPe, sizeof(aPe), &dwRead, NULL);
        CloseHandle(handle);

        if (bi.biSize != sizeof(BITMAPINFOHEADER))
            n = 0;
        else if (bi.biBitCount > 8)
            n = 0;
        else if (bi.biClrUsed == 0)
            n = 1 << bi.biBitCount;
        else
            n = bi.biClrUsed;
        
        /*  a DIB color table has its colors stored BGR not RGB
            so flip them around. */
        
        s = 0; e = n;
        if ((!pdxcContext->bFullScreen)&&(n==256))
        {
            s=10; e=246;
        }

        for(i=s; i<e; i++ )
        {
            pdxcContext->aPe[i].peRed  = aPe[i].peBlue;
            pdxcContext->aPe[i].peGreen = aPe[i].peGreen;
            pdxcContext->aPe[i].peBlue = aPe[i].peRed;
        }
    }
    else
        return FALSE;
    
    return TRUE;
}

BOOL DX_SetPalette(DX_CONTEXT *pdxcContext)
{
    if (pdxcContext->lpddpPalette)
    {
        if (pdxcContext->bFullScreen)
            pdxcContext->hLastResult = IDirectDrawPalette_SetEntries(pdxcContext->lpddpPalette, 0, 0, 256, pdxcContext->aPe);
        else
            pdxcContext->hLastResult = IDirectDrawPalette_SetEntries(pdxcContext->lpddpPalette, 0, 10, 236, &(pdxcContext->aPe[10]));
        if (FAILED(pdxcContext->hLastResult))
            RETURN_DDE_FALSE
        return TRUE;
    }
    else
        return FALSE;
}
