/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information

*/

#include <string.h>
#include <stdio.h>

#include "redx.h"
#include "pack.h"

#define PB_CHUNK 16384; /* number of pixels allocated in buffer */

typedef struct _PIXEL_BUFFER8
{
    DWORD dwX;
    DWORD dwY;
    BYTE bValue;
} PIXEL_BUFFER8;

DX_BITMAP *DXBitmap_ReloadPcx(DX_BITMAP *pdxbBitmap, DX_CONTEXT *pdxcContext, LPSTR lpFilename, BOOL bSetPalette, BOOL bVideo)
{
    PACKFILE *f;
    INT iC;
    INT iWidth, iHeight;
    DWORD dwBytesPerLine, dwX, dwY, dwNumPixels, dwBufferSize, dwColor;
    DWORD adwLocked[20];
    CHAR ch;
    PIXEL_BUFFER8 *pBuffer;
    PALETTEENTRY aPe[256];
    INT iFrom, iTo;

    if (pdxbBitmap)
    {
        pdxcContext = pdxbBitmap->pdxcContext;
        lpFilename = pdxbBitmap->lpName;
    }

    f = pack_fopen(lpFilename, F_READ);
    if (!f)
        return NULL;
    
    pack_getc(f);                    /* skip manufacturer ID */
    pack_getc(f);                    /* skip version flag */
    pack_getc(f);                    /* skip encoding flag */
    
    if (pack_getc(f) != 8)           /* we can only handle 8 bits per pixel */
    {
        pack_fclose(f);
        return NULL;
    }

    iWidth = -(pack_igetw(f));        /* xmin */
    iHeight = -(pack_igetw(f));       /* ymin */
    iWidth += pack_igetw(f) + 1;      /* xmax */
    iHeight += pack_igetw(f) + 1;     /* ymax */
    
    if (!pdxbBitmap)
    {
        pdxbBitmap = DX_CreateDXBitmap(pdxcContext, (DWORD)iWidth, (DWORD)iHeight, bVideo);
        pdxbBitmap->lpName = (char *)_strdup(lpFilename);
        pdxbBitmap->dxbType = pcx;
    }

    if (!pdxbBitmap)
    {
        pack_fclose(f);
        return FALSE;
    }
    DXBitmap_Clear(pdxbBitmap,DX_MakeColor(pdxcContext,0,0,0));
    
    pack_igetl(f);                   /* skip DPI values */
    
    for (iC=0; iC<16; iC++)          /* skip the 16 color pallete */
    {
        pack_getc(f);
        pack_getc(f);
        pack_getc(f);
    }
    
    pack_getc(f);
    
    if (pack_getc(f) != 1)           /* must be a 256 color file */
    {
        pack_fclose(f);
        DXBitmap_Free(pdxbBitmap);
        return NULL;
    }
    
    dwBytesPerLine = (DWORD)pack_igetw(f);
    
    for (iC=0; iC<60; iC++)          /* skip some more junk */
        pack_getc(f);

    dwBufferSize = PB_CHUNK;
    if (!(pBuffer = (PIXEL_BUFFER8 *)malloc(sizeof(PIXEL_BUFFER8)*dwBufferSize))) /* allocate buffer */
    {
        pack_fclose(f);
        DXBitmap_Free(pdxbBitmap);
        return NULL;
    }
    dwNumPixels = 0;
    
    for (dwY=0; dwY<(DWORD)iHeight; dwY++)       /* put RLE encoded PCX data in temporary buffer */
    {
        dwX = 0;
        
        while (dwX < dwBytesPerLine)
        {
            ch = pack_getc(f);
            if ((ch & 0xC0) == 0xC0)
            {
                iC = (ch & 0x3F);
                ch = pack_getc(f);
            }
            else
                iC = 1;
            
            while (iC--)
            {
                if (dwNumPixels==dwBufferSize)
                {
                    dwBufferSize += PB_CHUNK;
                    pBuffer = (PIXEL_BUFFER8 *)realloc(pBuffer,sizeof(PIXEL_BUFFER8)*dwBufferSize);
                }
                pBuffer[dwNumPixels].dwX = dwX++;
                pBuffer[dwNumPixels].dwY = dwY;
                pBuffer[dwNumPixels].bValue=(BYTE)ch;
                dwNumPixels++;
            }
        }
    }
    
    while (!pack_feof(f))           /* read the 256 color pallete */
    {
        if (pack_getc(f)==12)
        {
            for (iC=0; iC<256; iC++)
            {
                aPe[iC].peRed = pack_getc(f);
                aPe[iC].peGreen = pack_getc(f);
                aPe[iC].peBlue = pack_getc(f);
            }
            break;
        }
    }
    
    pack_fclose(f);
    
    if (errno)
    {
        DXBitmap_Free(pdxbBitmap);
        return NULL;
    }

    iFrom = (pdxcContext->bFullScreen?0:10); /* begin & end index of palette in case of 8 bit mode */
    iTo = (pdxcContext->bFullScreen?256:246);

    if ((pdxcContext->dwDepth==8)&&(bSetPalette)) /* handle the palette in 8 bit modes */
    {
        /* copy (relevant part of) the palette */
        for (iC=iFrom;iC<iTo;iC++)
        {
            pdxcContext->aPe[iC].peRed = aPe[iC].peRed;
            pdxcContext->aPe[iC].peGreen = aPe[iC].peGreen;
            pdxcContext->aPe[iC].peBlue = aPe[iC].peBlue;
        }
        DX_SetPalette(pdxcContext);
    }

    /* calculate the 20 'locked' colors if needed */
    if ((pdxcContext->dwDepth==8)&&(!(pdxcContext->bFullScreen)))
    {
        for (iC=0;iC<10;iC++)
            adwLocked[iC] = DX_MakeColor(pdxcContext,aPe[iC].peRed,aPe[iC].peGreen,aPe[iC].peBlue);
        for (iC=246;iC<256;iC++)
            adwLocked[iC-236] = DX_MakeColor(pdxcContext,aPe[iC].peRed,aPe[iC].peGreen,aPe[iC].peBlue);
    }

    if (DXBitmap_Lock(pdxbBitmap))
    {
        while (dwNumPixels--)
        {
            iC = pBuffer[dwNumPixels].bValue;
            if (pdxcContext->dwDepth==8)
                if ((iC>=iFrom)&&(iC<iTo))
                    dwColor = iC;
                else
                    dwColor = adwLocked[iC-(iC>=iTo?236:0)];
            else
                dwColor = DX_MakeColor(pdxcContext,aPe[iC].peRed,aPe[iC].peGreen,aPe[iC].peBlue);
            DXBitmap_PutPixel(pdxbBitmap,pBuffer[dwNumPixels].dwX,pBuffer[dwNumPixels].dwY,
                dwColor);
        }
        DXBitmap_Unlock(pdxbBitmap);
    }

    free(pBuffer);

    return pdxbBitmap;
}

DX_BITMAP *DX_LoadDXBitmapPcx(DX_CONTEXT *pdxcContext, LPSTR lpFilename, BOOL bSetPalette, BOOL bVideo)
{
    return DXBitmap_ReloadPcx(NULL,pdxcContext,lpFilename,bSetPalette,bVideo);
}

