/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information

*/

#include "redx.h"

#define STYLE_POP WS_POPUP
#define STYLE_WIN WS_OVERLAPPEDWINDOW|WS_SYSMENU

#define EXSTYLE_POP WS_EX_TOPMOST
#define EXSTYLE_WIN WS_EX_APPWINDOW

LRESULT CALLBACK RedxWndProc(HWND hWnd, UINT msg, WPARAM wParam,
                             LPARAM lParam);

int PASCAL WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR lpszCmdLine,
                   int nCmdShow );

static void UpdateRectClient(DX_CONTEXT *pdxcContext);
static BOOL StyleDXWindowPop(DX_CONTEXT *pdxcContext);
static BOOL StyleDXWindowWin(DX_CONTEXT *pdxcContext, int iWidth, int iHeight);

LRESULT CALLBACK RedxWndProc(HWND hWnd, UINT msg, WPARAM wParam,
                             LPARAM lParam)
{
    
    switch( msg )
    {
    case WM_SIZE:
    case WM_MOVE:
        {
            DX_CONTEXT *pdxcContext;
            pdxcContext = (DX_CONTEXT *)GetWindowLong(hWnd, GWL_USERDATA);
            if ((pdxcContext)&&(!pdxcContext->bFullScreen))
                UpdateRectClient(pdxcContext);
            break;
        }
    case WM_DESTROY:
        PostQuitMessage(0);
        break;

    case WM_SETCURSOR:
        {
            DX_CONTEXT *pdxcContext;
            pdxcContext = (DX_CONTEXT *)GetWindowLong(hWnd, GWL_USERDATA);
            if (pdxcContext)
                if (pdxcContext->bFullScreen)
                    SetCursor(NULL);
                else
                    SetCursor(LoadCursor(NULL,IDC_ARROW));
        }
        break;

    case WM_ACTIVATEAPP:
        {
            DX_CONTEXT *pdxcContext;
            pdxcContext = (DX_CONTEXT *)GetWindowLong(hWnd, GWL_USERDATA);
            pdxcContext->bActive = (BOOL)wParam;
            if (pdxcContext->bActive)
            {
                if ((pdxcContext->pdxbScreen)&&(pdxcContext->pdxbScreen->lpDDSPrimary))
                {
                    if (IDirectDrawSurface_IsLost(pdxcContext->pdxbScreen->lpDDSPrimary)==DDERR_SURFACELOST)
                    {
                        pdxcContext->hLastResult = IDirectDrawSurface_Restore(pdxcContext->pdxbScreen->lpDDSPrimary);
                        if (FAILED(pdxcContext->hLastResult))
                            PostQuitMessage(0);
                    }
                    if (!pdxcContext->bFullScreen)
                    {
                        if (IDirectDrawSurface_IsLost(pdxcContext->pdxbScreen->lpDDSBack)==DDERR_SURFACELOST)
                        {
                            pdxcContext->hLastResult = IDirectDrawSurface_Restore(pdxcContext->pdxbScreen->lpDDSBack);
                            if (FAILED(pdxcContext->hLastResult))
                                PostQuitMessage(0);
                        }
                    }
                }
            }
            REDXActivateApp(pdxcContext);
        }
        break;
        
    default:
        return( DefWindowProc( hWnd, msg, wParam, lParam ));
    }
    
    return 0;
}

int PASCAL WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR lpszCmdLine,
                   int nCmdShow )
{
    MSG msg;

    DX_CONTEXT *pdxcContext=NULL;

    pdxcContext = (DX_CONTEXT *)malloc(sizeof(DX_CONTEXT));
    memset(pdxcContext, 0, sizeof(DX_CONTEXT));

    pdxcContext->hinstApp = hInstance;

    if (REDXInit(pdxcContext))
    {
        while (1)
        {
            if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
            {
                if(!GetMessage(&msg, NULL, 0, 0 ))
                {
                    REDXExit(pdxcContext);
                    return msg.wParam;
                }
                TranslateMessage(&msg); 
                DispatchMessage(&msg);
            }
            else
            {
                if (!REDXLoop(pdxcContext))
                {
                    REDXExit(pdxcContext);
                    return 0;
                }
            }
        }
    }

    REDXExit(pdxcContext);
    return -1;
}

BOOL DX_Init(DX_CONTEXT *pdxcContext, LPSTR lpAppName, LPSTR lpAppTitle)
{
    pdxcContext->lpAppName = (LPSTR)malloc((strlen(lpAppName)+1)*sizeof(char));
    strcpy(pdxcContext->lpAppName, lpAppName);

    pdxcContext->lpAppTitle = (LPSTR)malloc((strlen(lpAppTitle)+1)*sizeof(char));
    strcpy(pdxcContext->lpAppTitle, lpAppTitle);

    pdxcContext->hLastResult = DirectDrawCreate(NULL, &(pdxcContext->lpDD), NULL);
    if (FAILED(pdxcContext->hLastResult))
        RETURN_DDE_FALSE

    return TRUE;
}

BOOL DX_SetMode(DX_CONTEXT *pdxcContext, BOOL bFullScreen,
                DWORD dwWidth, DWORD dwHeight, DWORD dwDepth)
               
{
    DDSURFACEDESC ddsd;
    DDSCAPS ddscaps;
    BOOL bReturn=FALSE;
    HDC hdc;
    LPDIRECTDRAWCLIPPER lpddClipper;
    DWORD dwX, dwY;
    RECT rect;

    /* if a mode was set, 'unset' it*/
    DX_UnSetMode(pdxcContext);

    /* if the window doesn't exist yet, create it */
    if (!(pdxcContext->hwndMain))
    {
        WNDCLASS wc;
        HBRUSH hbrush;

        if (bFullScreen)
            hbrush = (HBRUSH)(GetStockObject(NULL_BRUSH));
        else
            hbrush = (HBRUSH)((HBRUSH)(COLOR_WINDOW+1));

        wc.lpszClassName = pdxcContext->lpAppName;
        wc.lpfnWndProc = RedxWndProc;
        wc.style = CS_HREDRAW | CS_VREDRAW;
        wc.hInstance = pdxcContext->hinstApp;
        wc.hIcon = LoadIcon(pdxcContext->hinstApp, pdxcContext->lpAppName);
        wc.hCursor = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = hbrush;
        wc.lpszMenuName = pdxcContext->lpAppName;
        wc.cbClsExtra = 0;
        wc.cbWndExtra = 0;
        
        RegisterClass(&wc);

        if (!bFullScreen)
        {
            SetRect(&rect,0,0,dwWidth,dwHeight);
            AdjustWindowRectEx(&rect, STYLE_WIN, FALSE, EXSTYLE_WIN);
            dwX = rect.right - rect.left;
            dwY = rect.bottom - rect.top;
        }

        pdxcContext->hwndMain = CreateWindowEx(
            (bFullScreen?EXSTYLE_POP:EXSTYLE_WIN),
            pdxcContext->lpAppName,
            pdxcContext->lpAppTitle,
            (bFullScreen?STYLE_POP:STYLE_WIN),
            0,0,
            (bFullScreen?GetSystemMetrics(SM_CXSCREEN):dwX),
            (bFullScreen?GetSystemMetrics(SM_CYSCREEN):dwY),
            NULL,
            NULL,
            pdxcContext->hinstApp,
            NULL);

        if (!(pdxcContext->hwndMain))
            goto DX_SetMode_end;

        /* let the window userdata point to DX_CONTEXT */
        SetWindowLong(pdxcContext->hwndMain, GWL_USERDATA, (LONG)(pdxcContext));

        ShowWindow(pdxcContext->hwndMain, SW_SHOW);
    }
    else
    {
        /* otherwise set the right style */
        if (bFullScreen)
            StyleDXWindowPop(pdxcContext);
        else
            StyleDXWindowWin(pdxcContext, (int)dwWidth, (int)dwHeight);
    }

    pdxcContext->hLastResult = IDirectDraw_SetCooperativeLevel(pdxcContext->lpDD, pdxcContext->hwndMain,
        bFullScreen?DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN:DDSCL_NORMAL);

    if (FAILED(pdxcContext->hLastResult))
    {
        DD_ERROR
        goto DX_SetMode_end;
    }

    if (bFullScreen)
        pdxcContext->dwDepth = dwDepth;
    else
    {
        hdc = GetDC(0);
        pdxcContext->dwDepth = GetDeviceCaps(hdc,PLANES) * GetDeviceCaps(hdc,BITSPIXEL);
        ReleaseDC(0, hdc);
    }

    if (bFullScreen)
        if (FAILED((pdxcContext->hLastResult = IDirectDraw_SetDisplayMode(pdxcContext->lpDD, dwWidth,dwHeight,pdxcContext->dwDepth))))
        {
            DD_ERROR
            goto DX_SetMode_end;
        }


    if (!(pdxcContext->pdxbScreen))
    {
        pdxcContext->pdxbScreen = (DX_BITMAP *)malloc(sizeof(DX_BITMAP));
        memset(pdxcContext->pdxbScreen, 0, sizeof(DX_BITMAP));
        pdxcContext->pdxbScreen->pdxcContext = pdxcContext; /* got this one ? :-) */
        pdxcContext->pdxbScreen->dwWidth = dwWidth;
        pdxcContext->pdxbScreen->dwHeight = dwHeight;
        pdxcContext->pdxbScreen->bHardFlip = bFullScreen;
    }

    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS |(bFullScreen?DDSD_BACKBUFFERCOUNT:0);
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE|(bFullScreen?DDSCAPS_FLIP|DDSCAPS_COMPLEX:0);
    ddsd.dwBackBufferCount = bFullScreen?1:0;
    pdxcContext->hLastResult = IDirectDraw_CreateSurface(pdxcContext->lpDD, &ddsd,
        &(pdxcContext->pdxbScreen->lpDDSPrimary), NULL);
    if (FAILED(pdxcContext->hLastResult))
    {
        DD_ERROR
        goto DX_SetMode_end;
    }

    if (bFullScreen)
    {
        ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
        pdxcContext->hLastResult = IDirectDrawSurface_GetAttachedSurface(pdxcContext->pdxbScreen->lpDDSPrimary,
            &ddscaps, &(pdxcContext->pdxbScreen->lpDDSBack));
        if(FAILED(pdxcContext->hLastResult))
        {
            DD_ERROR
            goto DX_SetMode_end;
        }
    }
    else
    {
        ddsd.dwSize = sizeof(DDSURFACEDESC);
        ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
        ddsd.dwWidth  = dwWidth;
        ddsd.dwHeight = dwHeight;
        ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN
            | DDSCAPS_VIDEOMEMORY;
        
        pdxcContext->hLastResult = IDirectDraw_CreateSurface(pdxcContext->lpDD, &ddsd,
            &(pdxcContext->pdxbScreen->lpDDSBack), NULL);
        if (FAILED(pdxcContext->hLastResult))
        {
            ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN
             | DDSCAPS_SYSTEMMEMORY;
    
            pdxcContext->hLastResult = IDirectDraw_CreateSurface(pdxcContext->lpDD, &ddsd,
                &(pdxcContext->pdxbScreen->lpDDSBack), NULL);
            if (FAILED(pdxcContext->hLastResult))
            {
                DD_ERROR
                goto DX_SetMode_end;
            }
        }
    }

    if (!bFullScreen)
    {
        pdxcContext->hLastResult = DirectDrawCreateClipper(0, &lpddClipper, NULL);
        if (FAILED(pdxcContext->hLastResult))
        {
            DD_ERROR
            goto DX_SetMode_end;
        }
        pdxcContext->hLastResult = IDirectDrawClipper_SetHWnd(lpddClipper, 0, pdxcContext->hwndMain);
        if (FAILED(pdxcContext->hLastResult))
        {
            DD_ERROR
            IDirectDrawClipper_Release(lpddClipper);
            goto DX_SetMode_end;
        }
        pdxcContext->hLastResult = IDirectDrawSurface_SetClipper(pdxcContext->pdxbScreen->lpDDSPrimary, lpddClipper);
        if (FAILED(pdxcContext->hLastResult))
        {
            DD_ERROR
            IDirectDrawClipper_Release(lpddClipper);
            goto DX_SetMode_end;
        }
        IDirectDrawClipper_Release(lpddClipper);
    }

    

    pdxcContext->ddpf.dwSize = sizeof(DDPIXELFORMAT);
    pdxcContext->hLastResult = IDirectDrawSurface_GetPixelFormat(pdxcContext->pdxbScreen->lpDDSPrimary, &(pdxcContext->ddpf));

    if (FAILED(pdxcContext->hLastResult))
    {
        DD_ERROR
        goto DX_SetMode_end;
    }

    pdxcContext->bFullScreen = bFullScreen;

    /* create a palette in case of 8 bit mode */
    if (pdxcContext->dwDepth==8)
        DX_CreatePalette(pdxcContext);

    UpdateRectClient(pdxcContext);

    bReturn = TRUE;
    
DX_SetMode_end:
    return bReturn;
}

static void UpdateRectClient(DX_CONTEXT *pdxcContext)
{
    RECT rect;
    POINT pUL, pBR;
    
    if (!pdxcContext->hwndMain)
        return;

    GetClientRect(pdxcContext->hwndMain,&rect);

    pUL.x = rect.left;
    pUL.y = rect.top;
    pBR.x = rect.right;
    pBR.y = rect.bottom;

    ClientToScreen(pdxcContext->hwndMain, &pUL);
    ClientToScreen(pdxcContext->hwndMain, &pBR);

    if (pdxcContext->pdxbScreen)
        SetRect(&(pdxcContext->pdxbScreen->rectClient), pUL.x, pUL.y, pBR.x, pBR.y);
}

static BOOL StyleDXWindowPop(DX_CONTEXT *pdxcContext)
{
    HBRUSH hbrush;

    if (!(pdxcContext->hwndMain))
        return FALSE;

    ShowWindow(pdxcContext->hwndMain, SW_MINIMIZE);

    SetWindowLong(pdxcContext->hwndMain, GWL_EXSTYLE, EXSTYLE_POP);
    SetWindowLong(pdxcContext->hwndMain, GWL_STYLE, STYLE_POP);

    hbrush = (HBRUSH)(GetStockObject(NULL_BRUSH));
    SetClassLong(pdxcContext->hwndMain, GCL_HBRBACKGROUND, (long)hbrush);

    return TRUE;
}

static BOOL StyleDXWindowWin(DX_CONTEXT *pdxcContext, int iWidth, int iHeight)
{
    HBRUSH hbrush;
    RECT rect;
    int iX,iY;
    long lS, lES, lM;

    if (!(pdxcContext->hwndMain))
        return FALSE;

    ShowWindow(pdxcContext->hwndMain, SW_MINIMIZE);

    SetWindowLong(pdxcContext->hwndMain, GWL_EXSTYLE, EXSTYLE_WIN);
    SetWindowLong(pdxcContext->hwndMain, GWL_STYLE, STYLE_WIN);

    hbrush = (HBRUSH)((HBRUSH)(COLOR_WINDOW+1));
    SetClassLong(pdxcContext->hwndMain, GCL_HBRBACKGROUND, (long)hbrush);

    ShowWindow(pdxcContext->hwndMain, SW_RESTORE);

    iX = (GetSystemMetrics(SM_CXSCREEN)-iWidth)/2;
    iY = (GetSystemMetrics(SM_CYSCREEN)-iHeight)/2;

    rect.left = 0;
    rect.top = 0;
    rect.right = iWidth;
    rect.bottom = iHeight;

    lS = GetWindowLong(pdxcContext->hwndMain, GWL_STYLE);
    lES = GetWindowLong(pdxcContext->hwndMain, GWL_EXSTYLE);
    lM = GetWindowLong(pdxcContext->hwndMain, GCL_MENUNAME);

    AdjustWindowRectEx(&rect, lS, lM, lES);

    iWidth = rect.right - rect.left;
    iHeight = rect.bottom - rect.top;

    SetWindowPos(pdxcContext->hwndMain, HWND_TOP, iX, iY, iWidth, iHeight, 0);

    return TRUE;
}

BOOL DX_InitInput(DX_CONTEXT *pdxcContext)
{
    BOOL bReturn=FALSE;
    if (!pdxcContext->lpDI)
    {
        pdxcContext->hLastResult = DirectInputCreate(pdxcContext->hinstApp, DIRECTINPUT_VERSION,
            &(pdxcContext->lpDI), NULL);
        if (SUCCEEDED(pdxcContext->hLastResult))
            bReturn=TRUE;
        else
        {
            DI_ERROR
            pdxcContext->lpDI=NULL;
        }
    }
    else
        bReturn=TRUE;
    return bReturn;
}

void DX_UnSetMode(DX_CONTEXT *pdxcContext)
{
    if ((pdxcContext)&&(pdxcContext->lpDD))
    {
        if (pdxcContext->bFullScreen)
            IDirectDraw_RestoreDisplayMode(pdxcContext->lpDD);
        if (pdxcContext->pdxbScreen)
        {
            if (pdxcContext->pdxbScreen->lpDDSPrimary)
                IDirectDrawSurface_Release(pdxcContext->pdxbScreen->lpDDSPrimary);
            if ((!pdxcContext->bFullScreen)&&(pdxcContext->pdxbScreen->lpDDSBack))
                IDirectDrawSurface_Release(pdxcContext->pdxbScreen->lpDDSBack);
            if (pdxcContext->lpddpPalette)
            {
                IDirectDrawPalette_Release(pdxcContext->lpddpPalette);
                pdxcContext->lpddpPalette = NULL;
            }
            if (pdxcContext->pdxbScreen)
            {
                free(pdxcContext->pdxbScreen);
                pdxcContext->pdxbScreen = NULL;
            }
            if (pdxcContext->pbRgb8)
            {
                free(pdxcContext->pbRgb8);
                pdxcContext->pbRgb8 = NULL;
            }
        }
    }
}

void DX_Exit(DX_CONTEXT *pdxcContext)
{
    if (pdxcContext)
    {
        free(pdxcContext->lpAppName);
        free(pdxcContext->lpAppTitle);
        DX_UnSetMode(pdxcContext);
        if (pdxcContext->lpDIKeyboardDevice)
        {
            IDirectInputDevice_Unacquire(pdxcContext->lpDIKeyboardDevice);
            IDirectInputDevice_Release(pdxcContext->lpDIKeyboardDevice);
        }
        if (pdxcContext->lpDI)
        {
            IDirectInput_Release(pdxcContext->lpDI);
        }
        if (pdxcContext->uiTimerRes)
            timeEndPeriod(pdxcContext->uiTimerRes);
        if (pdxcContext->lpDS)
            IDirectSound_Release(pdxcContext->lpDS);
        if(pdxcContext->lpDD)
            IDirectDraw_Release(pdxcContext->lpDD);
        free(pdxcContext);
    }
}
