/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information

*/

#include "redx.h"

BOOL DX_InitSound(DX_CONTEXT *pdxcContext)
{
    if (pdxcContext->lpDS)
        return TRUE;
        
    pdxcContext->hLastResult = DirectSoundCreate(NULL, &(pdxcContext->lpDS), NULL);
    
    if (FAILED(pdxcContext->hLastResult))
    {
        pdxcContext->lpDS = NULL;
        RETURN_DSE_FALSE
    }
    
    pdxcContext->hLastResult = IDirectSound_SetCooperativeLevel(pdxcContext->lpDS, pdxcContext->hwndMain, DSSCL_NORMAL);
    
    if (FAILED(pdxcContext->hLastResult))
    {
        IDirectSound_Release(pdxcContext->lpDS);
        pdxcContext->lpDS = NULL;
        RETURN_DSE_FALSE

    }
    return TRUE;
}

DX_SOUNDBUFFER *DX_CreateDXSoundBuffer(DX_CONTEXT *pdxcContext, unsigned long size, WAVEFORMATEX* wfmt, BOOL bHardWare)
{
    DX_SOUNDBUFFER *pdxsbBuffer = NULL;
    DSBUFFERDESC dsbdesc;
    
    if (!pdxcContext->lpDS)
    {
        return NULL;
    }

    pdxsbBuffer = (DX_SOUNDBUFFER *)malloc(sizeof(DX_SOUNDBUFFER));
    memset(pdxsbBuffer,0,sizeof(DX_SOUNDBUFFER));
    pdxsbBuffer->pdxcContext = pdxcContext;
    
    memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));
    dsbdesc.dwSize = sizeof(DSBUFFERDESC);
    dsbdesc.dwFlags = DSBCAPS_CTRLDEFAULT | DSBCAPS_GETCURRENTPOSITION2 ;
    if (bHardWare)
        dsbdesc.dwFlags |= DSBCAPS_LOCHARDWARE;
    else
        dsbdesc.dwFlags |= DSBCAPS_LOCSOFTWARE;
    dsbdesc.dwBufferBytes = size; 
    dsbdesc.lpwfxFormat = wfmt;

    pdxcContext->hLastResult = IDirectSound_CreateSoundBuffer(pdxcContext->lpDS, &dsbdesc, &(pdxsbBuffer->lpDSBuffer), 0);
    if (FAILED(pdxcContext->hLastResult)&&(bHardWare))
    {
        dsbdesc.dwFlags = DSBCAPS_CTRLDEFAULT | DSBCAPS_GETCURRENTPOSITION2 | DSBCAPS_LOCSOFTWARE;
        pdxcContext->hLastResult = IDirectSound_CreateSoundBuffer(pdxcContext->lpDS, &dsbdesc, &(pdxsbBuffer->lpDSBuffer), 0);
    }
    if (FAILED(pdxcContext->hLastResult))
    {
        free(pdxsbBuffer);
        DS_ERROR
        return NULL;
    }

    return pdxsbBuffer;
}

DX_SOUNDBUFFER *DX_LoadDXSoundBuffer(DX_CONTEXT *pdxcContext, LPCSTR lpSoundName, BOOL bHardWare)
{
    DX_SOUNDBUFFER *pdxsbSoundBuffer;
    HMMIO file;
    MMCKINFO parent;
    MMCKINFO child;
    WAVEFORMATEX wfmtx;
    
    file = mmioOpen((LPSTR)lpSoundName, (LPMMIOINFO)0, MMIO_READ | MMIO_ALLOCBUF);
    if (!file)
        return NULL;

    memset(&parent, 0, sizeof(MMCKINFO));
    parent.fccType = mmioFOURCC('W', 'A', 'V', 'E');

    if (mmioDescend(file, &parent, 0, MMIO_FINDRIFF))
    {
        mmioClose(file, 0);
        return NULL;
    }

    memset(&child, 0, sizeof(MMCKINFO));
    child.fccType = mmioFOURCC('f', 'm', 't', ' ');

    if (mmioDescend(file, &child, &parent, 0))
    {
        mmioClose(file, 0);
        return NULL;
    }

    if (mmioRead(file, (char*)&wfmtx, sizeof(WAVEFORMATEX)) != sizeof(WAVEFORMATEX))
    {
         mmioClose(file, 0);
         return NULL;
    }

    if (wfmtx.wFormatTag != WAVE_FORMAT_PCM)
    {
        mmioClose(file, 0);
        return NULL;
    }

    if (mmioAscend(file, &child, 0))
    {
        mmioClose(file, 0);
        return NULL;
    }

    child.ckid = mmioFOURCC('d', 'a', 't', 'a');

    if (mmioDescend(file, &child, &parent, MMIO_FINDCHUNK))
    {
        mmioClose(file, 0);
        return NULL;
    }

    pdxsbSoundBuffer = DX_CreateDXSoundBuffer(pdxcContext, child.cksize, &wfmtx, bHardWare);
    if (!pdxsbSoundBuffer)
    {
        mmioClose(file, 0);
        return NULL;
    }

    if (!DXSoundBuffer_Write(pdxsbSoundBuffer, file, child.cksize))
    {
        DXSoundBuffer_Free(pdxsbSoundBuffer);
        mmioClose(file, 0);
        return NULL;
    }

    mmioClose(file, 0);
    return pdxsbSoundBuffer;
}

void DXSoundBuffer_Free(DX_SOUNDBUFFER *pdxsbSoundBuffer)
{
    if ((!pdxsbSoundBuffer)||(!pdxsbSoundBuffer->lpDSBuffer))
        return;
    if (!pdxsbSoundBuffer->pdxcContext->lpDS)
        return;

    IDirectSound_Release(pdxsbSoundBuffer->lpDSBuffer);
    free(pdxsbSoundBuffer);
}

BOOL DXSoundBuffer_Play(DX_SOUNDBUFFER *pdxsbSoundBuffer, BOOL bLooping)
{
    DWORD dwFlags;
    DX_CONTEXT *pdxcContext;

    if ((!pdxsbSoundBuffer)||(!pdxsbSoundBuffer->lpDSBuffer))
    {
        return FALSE;
    }

    pdxcContext = pdxsbSoundBuffer->pdxcContext;

    dwFlags = bLooping ? DSBPLAY_LOOPING : 0;
    pdxcContext->hLastResult =
    IDirectSoundBuffer_Play(pdxsbSoundBuffer->lpDSBuffer, 0, 0, dwFlags);
    if (SUCCEEDED(pdxcContext->hLastResult))
        return TRUE;

    if (pdxcContext->hLastResult == DSERR_BUFFERLOST)
    {
        pdxcContext->hLastResult =
        IDirectSoundBuffer_Restore(pdxsbSoundBuffer->lpDSBuffer);
    }
    
    if (SUCCEEDED(pdxcContext->hLastResult))
    {
        pdxsbSoundBuffer->pdxcContext->hLastResult =
        IDirectSoundBuffer_Play(pdxsbSoundBuffer->lpDSBuffer, 0, 0, dwFlags);
    }
    else
        DS_ERROR

    return (BOOL)(SUCCEEDED(pdxcContext->hLastResult));
}

BOOL DXSoundBuffer_Stop(DX_SOUNDBUFFER *pdxsbSoundBuffer)
{
    DX_CONTEXT *pdxcContext;

    if ((!pdxsbSoundBuffer)||(!pdxsbSoundBuffer->lpDSBuffer))
    {
        return FALSE;
    }

    pdxcContext = pdxsbSoundBuffer->pdxcContext;

    pdxcContext->hLastResult = IDirectSoundBuffer_Stop(pdxsbSoundBuffer->lpDSBuffer);
	if (FAILED(pdxcContext->hLastResult))
        RETURN_DSE_FALSE

    return TRUE;
}

BOOL DXSoundBuffer_Write(DX_SOUNDBUFFER *pdxsbSoundBuffer, HMMIO file, unsigned long size)
{
    DX_CONTEXT *pdxcContext;
    void* w1 = 0; unsigned long l1 = 0;
    void* w2 = 0; unsigned long l2 = 0;

    pdxcContext = pdxsbSoundBuffer->pdxcContext;

    pdxcContext->hLastResult =
    IDirectSoundBuffer_Lock(pdxsbSoundBuffer->lpDSBuffer, 0, size, &w1, &l1, &w2, &l2, 0);

	if (FAILED(pdxcContext->hLastResult))
        RETURN_DSE_FALSE

	if (w1)
	{
		if (mmioRead(file, (char*)w1, l1) != (long)l1)
		{
			return FALSE;
		}
	}

	if (w2)
	{
		if (mmioRead(file, (char*)w2, l2) != (long)l2)
		{
			return FALSE;
		}
	}

    pdxcContext->hLastResult =
    IDirectSoundBuffer_Unlock(pdxsbSoundBuffer->lpDSBuffer, w1, l1, w2, l2);

	if (FAILED(pdxcContext->hLastResult))
        RETURN_DSE_FALSE

	return TRUE;
}


DX_SOUNDBUFFER *DXSoundBuffer_Copy(DX_SOUNDBUFFER *pdxsbSoundBuffer)
{
    DX_CONTEXT *pdxcContext;
    DX_SOUNDBUFFER *pdxsbCopy=NULL;
    
	if ((!pdxsbSoundBuffer)||(!pdxsbSoundBuffer->lpDSBuffer))
	{
		return NULL;
	}

    pdxcContext = pdxsbSoundBuffer->pdxcContext;

    pdxsbCopy = (DX_SOUNDBUFFER *)malloc(sizeof(DX_SOUNDBUFFER));
    memset(pdxsbCopy,0,sizeof(DX_SOUNDBUFFER));
    pdxsbCopy->pdxcContext = pdxsbSoundBuffer->pdxcContext;

    pdxcContext->hLastResult =
    IDirectSound_DuplicateSoundBuffer(pdxsbSoundBuffer->pdxcContext->lpDS ,pdxsbSoundBuffer->lpDSBuffer, &(pdxsbCopy->lpDSBuffer));


	if (FAILED(pdxcContext->hLastResult))
	{
        DS_ERROR
	    free(pdxsbCopy);
		return NULL;
	}

	return pdxsbCopy;
}

BOOL DXSoundBuffer_SetPan(DX_SOUNDBUFFER *pdxsbSoundBuffer, LONG val)
{
    DX_CONTEXT *pdxcContext;
	if ((!pdxsbSoundBuffer)||(!pdxsbSoundBuffer->lpDSBuffer))
	{
		return FALSE;
	}

    pdxcContext = pdxsbSoundBuffer->pdxcContext;

    pdxcContext->hLastResult =
	IDirectSoundBuffer_SetPan(pdxsbSoundBuffer->lpDSBuffer, val);

	if (FAILED(pdxcContext->hLastResult))
        RETURN_DSE_FALSE

	return TRUE;
}

BOOL DXSoundBuffer_SetVolume(DX_SOUNDBUFFER *pdxsbSoundBuffer, LONG val)
{
    DX_CONTEXT *pdxcContext;
	if ((!pdxsbSoundBuffer)||(!pdxsbSoundBuffer->lpDSBuffer))
	{
		return FALSE;
	}

    pdxcContext = pdxsbSoundBuffer->pdxcContext;

    pdxcContext->hLastResult =
	IDirectSoundBuffer_SetVolume(pdxsbSoundBuffer->lpDSBuffer, val);

	if (FAILED(pdxcContext->hLastResult))
        RETURN_DSE_FALSE

	return TRUE;
}

DX_SOUND *DX_LoadDXSound(DX_CONTEXT *pdxcContext, LPCSTR lpSoundName, DWORD dwNumCopies, BOOL bHardWare)
{
    DX_SOUND *pdxsSound;
    INT i;

    /* create and empty structure */
    if (!(pdxsSound = (DX_SOUND *)malloc(sizeof(DX_SOUND))))
        return NULL;
    memset((void *)pdxsSound,0,sizeof(DX_SOUND));

    /* allocate array */
    pdxsSound->apdxsbSoundBuffer
        = (DX_SOUNDBUFFER **)malloc(sizeof(DX_SOUNDBUFFER *)*dwNumCopies);
    if (!(pdxsSound->apdxsbSoundBuffer))
    {
        free(pdxsSound);
        return NULL;
    }

    /* number of copies */
    pdxsSound->dwNumSounds = dwNumCopies;

    /* load 1 soundbuffer */
    if (!(pdxsSound->apdxsbSoundBuffer[0] = DX_LoadDXSoundBuffer(pdxcContext, lpSoundName, bHardWare)))
    {
        free(pdxsSound->apdxsbSoundBuffer);
        free(pdxsSound);
        return NULL;
    }

    /* create the copies */
    for (i=1;i<(int)pdxsSound->dwNumSounds;i++)
    {
        pdxsSound->apdxsbSoundBuffer[i] = DXSoundBuffer_Copy(pdxsSound->apdxsbSoundBuffer[0]);
        /* if no copy, try to load it again */
        if (!(pdxsSound->apdxsbSoundBuffer[i]))
            pdxsSound->apdxsbSoundBuffer[i] = DX_LoadDXSoundBuffer(pdxcContext, lpSoundName, bHardWare);
        /* if still no copy, fail */
        if (!(pdxsSound->apdxsbSoundBuffer[i]))
        {
            while (--i!=0)
                DXSoundBuffer_Free(pdxsSound->apdxsbSoundBuffer[i]);
            free(pdxsSound->apdxsbSoundBuffer);
            free(pdxsSound);
            return NULL;
        }
    }

    return (pdxsSound);
}

BOOL DXSound_Play(DX_SOUND *pdxsSound)
{
    DX_CONTEXT *pdxcContext;
    BOOL bReturn;

    /* check if the sound exists */
    if (!(pdxsSound))
        return FALSE;

    /* retrieve the context */
    pdxcContext = pdxsSound->apdxsbSoundBuffer[pdxsSound->dwCurrentSound]->pdxcContext;

    /* set the position of the buffer to 0 */
    pdxcContext->hLastResult = IDirectSoundBuffer_SetCurrentPosition(
        pdxsSound->apdxsbSoundBuffer[pdxsSound->dwCurrentSound]->lpDSBuffer,0);
	if (FAILED(pdxcContext->hLastResult))
        RETURN_DSE_FALSE
    
    
    /* play the current buffer */
    bReturn = DXSoundBuffer_Play(pdxsSound->apdxsbSoundBuffer[pdxsSound->dwCurrentSound++],FALSE);

    if (pdxsSound->dwCurrentSound>=pdxsSound->dwNumSounds)
        pdxsSound->dwCurrentSound = 0;

    return bReturn;
}

BOOL DXSound_SetPan(DX_SOUND *pdxsSound, LONG lPan)
{
    /* check if the sound exists */
    if (!(pdxsSound))
        return FALSE;

    /* set he pan of the current buffer */
    return DXSoundBuffer_SetPan(pdxsSound->apdxsbSoundBuffer[pdxsSound->dwCurrentSound], lPan);
}

BOOL DXSound_Setvolume(DX_SOUND *pdxsSound, LONG lVolume)
{
    /* check if the sound exists */
    if (!(pdxsSound))
        return FALSE;

    /* set the volume of the current buffer */
    return DXSoundBuffer_SetVolume(pdxsSound->apdxsbSoundBuffer[pdxsSound->dwCurrentSound], lVolume);
}

void DXSound_Free(DX_SOUND *pdxsSound)
{
    DWORD dwI;

    /* check if the sound exists */
    if (!(pdxsSound))
        return;

    /* free the soundbuffers */
    dwI = pdxsSound->dwNumSounds;
    while (dwI--)
        DXSoundBuffer_Free(pdxsSound->apdxsbSoundBuffer[dwI]);

    /* free the sound buffer array */
    if (pdxsSound->apdxsbSoundBuffer)
        free(pdxsSound->apdxsbSoundBuffer);

    /* free the structure */
    free(pdxsSound);
}
