/*
         _____          ___   __
        |  __ \        | \ \ / /
        | |__) |___  __| |\ V / 
        |  _  // _ \/ _` | > <  
        | | \ \  __/ (_| |/ . \ 
        |_|  \_\___|\__,_/_/ \_\

        A Win32 Game Programming
               Library
                 by
           Dominique Biesmans
     (c) 1998 - all rights reserved
see readme.1st for copyright information

*/

#include "redx.h"


DX_SPRITE *DX_CreateSprite(DX_CONTEXT *pdxcContext, DX_BITMAP *pdxbBitmap, DWORD dwNumFrames)
{
    DX_SPRITE *pdxsSprite;
    if ((dwNumFrames<1)||(!pdxbBitmap))
        return NULL;
    pdxsSprite = malloc(sizeof(DX_SPRITE));
    memset(pdxsSprite, 0, sizeof(DX_SPRITE));
    pdxsSprite->pdxbBitmap = pdxbBitmap;
    pdxsSprite->pRectArray = malloc(sizeof(RECT)*dwNumFrames);
    memset(pdxsSprite->pRectArray, 0 ,sizeof(RECT)*dwNumFrames);
    pdxsSprite->dwNumFrames = dwNumFrames;
    return (pdxsSprite);
}

DX_SPRITE *DX_CreateSpriteGrid(DX_CONTEXT *pdxcContext, DX_BITMAP *pdxbBitmap, DWORD dwNumFrames,
                               DWORD dwNumCols, DWORD dwNumRows)
{
    DX_SPRITE *pdxsSprite;
    DWORD i, j, dwFrame;
    RECT *pRect;
    if (pdxsSprite = DX_CreateSprite(pdxcContext, pdxbBitmap, dwNumFrames))
    {
        pdxsSprite->dwWidth = pdxsSprite->pdxbBitmap->dwWidth/dwNumCols;
        pdxsSprite->dwHeight = pdxsSprite->pdxbBitmap->dwHeight/dwNumRows;
        for (i=0;i<dwNumRows;i++)
            for (j=0;j<dwNumCols;j++)
            {
                dwFrame = i*dwNumCols+j;
                if (dwFrame>=pdxsSprite->dwNumFrames)
                {
                    i=dwNumRows;
                    break;
                }
                pRect = &(pdxsSprite->pRectArray[dwFrame]);
                pRect->left=j*pdxsSprite->dwWidth;
                pRect->top=i*pdxsSprite->dwHeight;
                pRect->right=pRect->left+pdxsSprite->dwWidth;
                pRect->bottom=pRect->top+pdxsSprite->dwHeight;
            }
    }
    return pdxsSprite;
}

BOOL DXSprite_MakeTrans(DX_SPRITE *pdxsSprite)
{
    return DXBitmap_MakeTrans(pdxsSprite->pdxbBitmap);
}

void DXSprite_Move(DX_SPRITE *pdxsSprite, FLOAT fX, FLOAT fY)
{
    pdxsSprite->fX = fX;
    pdxsSprite->fY = fY;
}

void DXSprite_SetFrame(DX_SPRITE *pdxsSprite, DWORD dwFrame)
{
    pdxsSprite->dwCurrentFrame = dwFrame;
}

void DXSprite_NextFrame(DX_SPRITE *pdxsSprite)
{
    pdxsSprite->dwCurrentFrame++;
    if (pdxsSprite->dwCurrentFrame>=pdxsSprite->dwNumFrames)
        pdxsSprite->dwCurrentFrame=0;
}

BOOL DXSprite_Draw(DX_SPRITE *pdxsSprite, DX_BITMAP *pdxbBitmap)
{
    return DXBitmap_BlitRect(pdxsSprite->pdxbBitmap, pdxbBitmap,
            &(pdxsSprite->pRectArray[pdxsSprite->dwCurrentFrame]),
            (LONG)pdxsSprite->fX, (LONG)pdxsSprite->fY);
}

void DXSprite_Free(DX_SPRITE *pdxsSprite)
{
    if (pdxsSprite)
    {
        if (pdxsSprite->pRectArray)
            free(pdxsSprite->pRectArray);
        free(pdxsSprite);
    }
}
